#!/usr/bin/python
#dhn.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys


def windows_and_frozen():
    import platform
    if platform.uname()[0] != 'Windows':
        # return False
        # under Linux we have same problem too ...
        # I have no idea!
        return True

    #http://www.py2exe.org/index.cgi/HowToDetermineIfRunningFromExe
    import imp
    return  (hasattr(sys, "frozen") or # new py2exe
            hasattr(sys, "importers") or# old py2exe
            imp.is_frozen("__main__")) # tools/freeze



if __name__ == "__main__":

    how_to_stop = windows_and_frozen()

    import p2p.dhnmain
    ret = p2p.dhnmain.main()

    # we use py2exe to build DataHaven.NET under Windows
    # but it wont finish correctly!
    # after finishing all process dhnmain.exe still working
    # and eat 50% CPU! - it wont stop.
    # but if you running from command line - it gives no problems
    # ... it seems CSpace reactor make some conflicts with twisted reactor
    # however all threads are finished correctly - really strange thing ...
    # TODO I definately should find where is the problem here.
    # but let's take this for now ... it is working.
    
    if how_to_stop:
        os._exit(ret)
    else:
        sys.exit(ret)




#!/usr/bin/python
#transport_udp_session.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#


import os
import time
import struct
import cStringIO
import random


from twisted.internet import reactor
from twisted.internet.task import LoopingCall 


import dhnio
import misc
import identity
import identitycache
import tmpfile 
import nameurl

import automat
import automats
import transport_udp

#------------------------------------------------------------------------------ 

CODES = {'p:':  'PING',
         'g:':  'GREETING',
         'd:':  'DATA',
         'r:':  'REPORT',
         'a:':  'ALIVE',}

COMMANDS = {'PING':         'p:',
            'GREETING':     'g:',
            'DATA':         'd:',
            'REPORT':       'r:',
            'ALIVE':        'a:',}

BLOCK_SIZE_LEVELS = {   #0: 508,
                        0: 1460,
                        1: 2048,
                        2: 3072,
                        3: 4096,
                        4: 5120,
                        5: 6144,
                        6: 7168,
                        7: 8124,
                        }
MAX_BLOCK_SIZE_LEVEL = len(BLOCK_SIZE_LEVELS) - 1

#BLOCK_SIZE = 4096
TIME_OUT = 20
BLOCK_RETRIES = 5
MAX_WINDOW_SIZE = 32
MIN_WINDOW_SIZE = 1

_ParentObject = None
_SessionsDict = {}
_IdToAddress = {}
_AddressToId = {}

_SendStatusCallback = None
_ReceiveStatusCallback = None
_SendControlFunc = None
_ReceiveControlFunc = None

_Timer1sec = None
_Timer10sec = None
_Timer30sec = None

#------------------------------------------------------------------------------ 

def init(parent):
    global _ParentObject
    dhnio.Dprint(4, 'transport_udp_session.init')
    _ParentObject = parent
   
def A(address, event=None, arg=None):
    global _SessionsDict
    s = open_session(address, arg)
    if event is not None:
        s.automat(event, arg)
    return s
      

def open_session(address, arg):
    global _SessionsDict
    if _SessionsDict.has_key(address):
        return _SessionsDict[address]
    idurl = None
    if isinstance(arg, str) and arg.startswith('http://'):
        idurl = arg
    name = 'udp_%s_(%s:%s)' % (nameurl.GetName(idurl), address[0], address[1])
    s = TransportUDPSession(address, name, 'AT_STARTUP', 12)
    s.event('init', idurl)
    _SessionsDict[address] = s
    return s
    

class TransportUDPSession(automat.Automat):
    timers = {#'timer-1min':     (60,    ['CONNECTED', 'SENDING']),
              #'timer-30sec':    (30,    ['GREETING', 'PING', 'DISCONNECTED']),
              #'timer-1sec':     (1,     ['PING', 'GREETING', 'CONNECTED']),
              'timer-01sec':    (0.1,   ['SENDING', 'WAITING'])}
    def __init__(self, address, name, state, debug_level):
        global _ParentObject
        self.remote_address = address
        self.remote_idurl = None
        self.parent = _ParentObject
        self.client = _ParentObject.client
        self.last_alive_packet_time = 0
        self.incommingFiles = {}
        self.outgoingFiles = {}
        self.receivedFiles = {}
        self.outbox_queue = []
        self.tries = 0
        self.current_filename = ''
        self.current_file_id = None
        self.current_block_id = -1
        self.current_blocks = 0
        self.sliding_window = {}
        self.sliding_window_size = MIN_WINDOW_SIZE
        self.last_bytes_sent = 0
        self.failedBlocks = {}
        self.hasFailedBlocks = False
        self.block_size_level = 0
        automat.Automat.__init__(self, name, state, debug_level)
        
    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'init':
                self.doSaveIDURL(arg)
                self.doRequestRemoteID()
                self.state = 'REMOTE_ID'
        #---SENDING---
        elif self.state == 'SENDING':
            if event == 'timer-01sec' and self.isMoreBlocks() and self.isWindowOpen():
                self.doSendBlocks() 
                self.doCheckWindow()               
            elif ( event == 'timer-01sec' and not self.isFailedBlocks() ) and ( not self.isWindowOpen() or ( not self.isMoreBlocks() and not self.isAllSent() ) ):
                self.doCheckWindow()
            elif ( event == 'timer-01sec' and self.isFailedBlocks() ) or ( event == 'timer-30sec' and not self.isAlive() ):
                self.doReportFailed()
                self.doCloseFile()
                self.state = 'CONNECTED'
            elif event == 'timer-01sec' and not self.isMoreBlocks() and self.isAllSent():
                self.doReportDone()
                self.doCloseFile()
                self.state = 'CONNECTED'
            elif event == 'datagram-received':
                self.doReceiveData(arg)
            elif event == 'shutdown':
                self.doReportFailed()
                self.doCloseFile()
                self.doDestroyMe()
                self.state = 'CLOSED'
        #---REMOTE_ID---
        elif self.state == 'REMOTE_ID':
            if event == 'remote-id-received' and not self.isIPPortChanged():
                self.doPing()
                self.state = 'PING'
            elif event == 'remote-id-received' and self.isIPPortChanged():
                self.parent.A('ip-port-changed', self.remote_idurl)
                self.doDestroyMe()
                self.state = 'CLOSED'
            elif event == 'remote-id-failed':
                self.state = 'DISCONNECTED'
            elif event == 'datagram-received':
                self.doReceiveData(arg)
            elif event == 'shutdown':                
                self.doDestroyMe()
                self.state = 'CLOSED'
        #---PING---
        elif self.state == 'PING':
            if event == 'datagram-received' and self.isPingOrGreeting(arg):
                self.doGreeting()
                self.state = 'GREETING'
            elif event == 'datagram-received' and not self.isPingOrGreeting(arg):
                self.doReceiveData(arg)
            elif event == 'timer-1sec':
                self.doPing()
            elif event == 'timer-30sec':
                self.state = 'DISCONNECTED'
            elif event == 'shutdown':
                self.doDestroyMe()
                self.state = 'CLOSED'
        #---GREETING---
        elif self.state == 'GREETING':
            if event == 'datagram-received' and self.isGreetingOrAlive(arg):
                self.doAlive()
                self.state = 'CONNECTED'
            elif event == 'datagram-received' and not self.isGreetingOrAlive(arg):
                self.doReceiveData(arg)
            elif event == 'timer-1sec':
                self.doGreeting()
            elif  event == 'timer-30sec':
                self.state = 'DISCONNECTED'
            elif event == 'shutdown':
                self.doDestroyMe()
                self.state = 'CLOSED'
        #---CONNECTED---
        elif self.state == 'CONNECTED':
            if event == 'datagram-received':
                self.doReceiveData(arg)
            elif event == 'timer-1sec' and self.isOutgoingFiles():
                self.doReadFile()
                self.tries = 0
                self.state = 'SENDING'
            elif event == 'timer-30sec' and self.isAlive():
                self.doAlive()
            elif event == 'timer-30sec' and not self.isAlive():
                self.state = 'DISCONNECTED'
            elif event == 'shutdown':
                self.doDestroyMe()
                self.state = 'CLOSED'
        #---DISCONNECTED---
        elif self.state == 'DISCONNECTED':
            if event == 'timer-10sec':
                self.doRequestRemoteID() 
                self.state = 'REMOTE_ID'
            elif event == 'datagram-received':
                self.doPing()
                self.state = 'PING'
            elif event == 'shutdown':
                self.doDestroyMe()
                self.state = 'CLOSED'
        #---CLOSED---
        elif self.state == 'CLOSED':
            pass
            
    def isPingOrGreeting(self, arg):
        return arg[0][:2] in [ COMMANDS['PING'], COMMANDS['GREETING'], ] 
            
    def isGreetingOrAlive(self, arg):
        return arg[0][:2] in [ COMMANDS['GREETING'], COMMANDS['ALIVE'], ]
    
    def isAlive(self):
        return time.time() - self.last_alive_packet_time < 60.0 * 1.0 + 5.0
    
    def isOutgoingFiles(self):
#        if len(self.outbox_queue) == 0:
#            return
#        for filename in self.outbox_queue:
#            if not os.path.isfile(filename):
#                continue
#            if not os.access(filename, os.R_OK):
#                continue
#            return True
#        return False
        return len(self.outbox_queue) > 0
    
    def isAllSent(self):
        return len(self.outgoingFiles[self.current_file_id]) == 0
    
    def isMoreBlocks(self):
        return self.current_block_id < self.current_blocks
    
    def isIPPortChanged(self):
        if self.remote_idurl is None:
            return False
        ident = identitycache.FromCache(self.remote_idurl)
        new_address = ident.getProtoContact('udp')
        if new_address is None:
            return True
        try:
            ip, port = new_address[6:].split(':')
            new_address = (ip, int(port))
        except:
            return True
        new_address = self.parent.remapAddress(new_address, self.remote_idurl)
        return self.remote_address != new_address                
    
    def isWindowOpen(self):
        return len(self.sliding_window) < self.sliding_window_size
    
    def isFailedBlocks(self):
        return self.hasFailedBlocks
    
    def doSaveIDURL(self, arg):
        self.remote_idurl = arg
    
    def doRequestRemoteID(self):
        if self.remote_idurl is None:
            self.automat('remote-id-received')
        else:
            identitycache.immediatelyCaching(self.remote_idurl).addCallbacks(
                lambda src: self.automat('remote-id-received', src),
                lambda x: self.automat('remote-id-failed', x))
    
    def doPing(self):
        self.sendDatagram(COMMANDS['PING'])
    
    def doGreeting(self):
        self.sendDatagram(COMMANDS['GREETING']+misc.getLocalID())
    
    def doAlive(self):
        self.sendDatagram(COMMANDS['ALIVE'])
    
    def doReceiveData(self, arg):
        global _ReceiveControlFunc
        io = cStringIO.StringIO(arg[0])
        code = io.read(2)
        cmd = CODES.get(code, None)
        if cmd is None:
            # dhnio.Dprint(2, 'transport_udp_session.doReceiveData WARNING incorrect data from %s: [%s]' % (self.remote_address, code))
            return
        if _ReceiveControlFunc is not None:
            seconds_pause = _ReceiveControlFunc(len(arg[0]))
            if seconds_pause > 0:
                # print 'paused for', seconds_pause, 'seconds'
                self.client.transport.pauseProducing()
                reactor.callLater(seconds_pause, self.client.transport.resumeProducing)
        # dhnio.Dprint(8, '                                UDP:[%s] from %s' % (cmd, self.remote_address))
        self.last_alive_packet_time = time.time()
        #--- DATA
        if cmd == 'DATA':
            try:
                file_id = struct.unpack('i', io.read(4))[0]
                block_id = struct.unpack('i', io.read(4))[0]
                num_blocks = struct.unpack('i', io.read(4))[0]
                data_size = struct.unpack('i', io.read(4))[0]
            except:
                dhnio.DprintException()
                return
            if not self.incommingFiles.has_key(file_id):
                if self.receivedFiles.has_key(file_id):
                    self.sendDatagram(COMMANDS['REPORT'] + struct.pack('i', file_id) + struct.pack('i', block_id))
                    return
                self.incommingFiles[file_id] = {}
            inp_data = io.read()
            if len(inp_data) != data_size:
                dhnio.Dprint(2, 'transport_udp_session.doReceiveData WARNING incorrect datagram received, not full data')
                return
            self.incommingFiles[file_id][block_id] = inp_data 
            self.sendDatagram(COMMANDS['REPORT'] + struct.pack('i', file_id) + struct.pack('i', block_id))
            # dhnio.Dprint(10, 'transport_udp_session.doReceiveData [%d] (%d/%d) from %s' % (file_id, block_id, num_blocks, self.remote_address))
            if len(self.incommingFiles[file_id]) == num_blocks:
                self.buildInboxFile(file_id)
                del self.incommingFiles[file_id]
                self.receivedFiles[file_id] = time.time()
                self.eraseOldFileIDs()
        #--- REPORT
        elif cmd == 'REPORT':
            try:
                file_id = struct.unpack('i', io.read(4))[0]
            except:
                dhnio.DprintException()
                return
            if not self.outgoingFiles.has_key(file_id):
                # dhnio.Dprint(2, 'transport_udp_session.doReceiveData WARNING unknown file_id in REPORT packet received from %s: [%d]' % (self.remote_address, file_id))
                return
            try:
                block_id = struct.unpack('i', io.read(4))[0]
            except:
                dhnio.DprintException()
                return
            if not self.outgoingFiles[file_id].has_key(block_id):
                # dhnio.Dprint(2, 'transport_udp_session.doReceiveData WARNING unknown block_id in REPORT packet received from %s: [%d]' % (self.remote_address, block_id))
                return
            del self.outgoingFiles[file_id][block_id]
            self.sliding_window.pop((file_id, block_id), None)
            self.sliding_window_size += 1
            if self.sliding_window_size > MAX_WINDOW_SIZE:
                self.sliding_window_size = MAX_WINDOW_SIZE
            # dhnio.Dprint(8, 'transport_udp_session.doReceiveData REPORT on [%d,%d] received, blocks=%d, window=%d' % (file_id, block_id, len(self.outgoingFiles[file_id]), self.sliding_window_size))
        #--- GREETING
        elif cmd == 'GREETING':
            greeting_idurl = io.read()
            if greeting_idurl:
                if self.remote_idurl is None:
                    self.remote_idurl = greeting_idurl
                    dhnio.Dprint(6, 'transport_udp_session.doReceiveData recognized remote idurl from GREETING: %s' % greeting_idurl)
                else:
                    if self.remote_idurl != greeting_idurl:
                        dhnio.Dprint(2, 'transport_udp_session.doReceiveData WARNING incorrect remote idurl in GREETING: %s' % greeting_idurl)
            self.sendDatagram(COMMANDS['ALIVE'])
        #--- PING
        elif cmd == 'PING':
            self.sendDatagram(COMMANDS['GREETING']+misc.getLocalID())
        #--- ALIVE            
        elif cmd == 'ALIVE':
            pass

    def doReadFile(self):
        self.current_filename = self.outbox_queue.pop(0)
        self.current_file_id = self.makeFileID()
        self.outgoingFiles[self.current_file_id] = {}
#        self.blocksReports[self.current_file_id] = 0
        fin = open(self.current_filename, 'rb')
        block_id = 0
        while True:
            block_data = fin.read(BLOCK_SIZE_LEVELS[self.block_size_level])
            if block_data == '':
                break
            self.outgoingFiles[self.current_file_id][block_id] = block_data
            block_id += 1    
        fin.close()     
        self.current_block_id = 0     
        self.current_blocks = block_id  
        # dhnio.Dprint(6, 'transport_udp_session.doReadFile file_id=%d, blocks=%d' % (
        #     self.current_file_id, len(self.outgoingFiles[self.current_file_id])))
    
    def doCheckWindow(self):
        if len(self.sliding_window) == 0:
            return
        # dhnio.Dprint(14, 'transport_udp_session.doCheckWindow has %d items' % len(self.sliding_window))
        to_remove = [] 
        to_update = {}
        cur_tm = time.time()
        for block_info, tm in self.sliding_window.items():
            if cur_tm - tm < TIME_OUT: # give some time to get an Ack. 
                continue
            to_remove.append(block_info)
            file_id = block_info[0]
            block_id = block_info[1]
            if file_id != self.current_file_id:
                continue
            if not self.outgoingFiles.has_key(file_id):
                dhnio.Dprint(2, 'transport_udp_session.doCheckWindow WARNING unknown file_id: %d' % file_id)
                continue
            if not self.outgoingFiles[file_id].has_key(block_id):
                dhnio.Dprint(2, 'transport_udp_session.doCheckWindow WARNING unknown block_id: %d' % block_id)
                continue
            self.sendBlock(block_id)
            isFailed = self.markFailedBlock(block_id)
            if not isFailed:
                to_update[block_info] = cur_tm
        for block_info in to_remove:
            del self.sliding_window[block_info]
            # dhnio.Dprint(8, 'transport_udp_session.doCheckWindow (%d,%d) removed, window=%d' % (file_id, block_id, len(self.sliding_window)))
        for block_info, tm in to_update.items():
            self.sliding_window[block_info] = tm
            # dhnio.Dprint(8, 'transport_udp_session.doCheckWindow (%s) resending to %s, window=%d' % (block_info, self.remote_address, len(self.sliding_window)))
        if len(to_remove) > 0:
            self.sliding_window_size = int( self.sliding_window_size / 2.0 )
            if self.sliding_window_size < MIN_WINDOW_SIZE:
                self.sliding_window_size = MIN_WINDOW_SIZE
            # dhnio.Dprint(8, 'transport_udp_session.doCheckWindow decreased, window=%d' % self.sliding_window_size)
        del to_remove
        del to_update
        
    def doSendBlocks(self):
        while len(self.sliding_window) < self.sliding_window_size and self.current_block_id < self.current_blocks:
            if not self.sendBlock(self.current_block_id):
                break
            self.sliding_window[(self.current_file_id, self.current_block_id)] = time.time()
            self.current_block_id += 1
    
    def doCloseFile(self):
        del self.outgoingFiles[self.current_file_id]
        self.failedBlocks.pop(self.current_file_id, None)
        # dhnio.Dprint(6, 'transport_udp_session.doCloseFile [%d] closed' % self.current_file_id)
        self.current_file_id = None
        self.current_filename = ''
        self.current_block_id = -1
        self.current_blocks = 0
        self.last_bytes_sent = 0
        self.hasFailedBlocks = False
        
    def doReportDone(self):    
        # dhnio.Dprint(6, 'transport_udp_session.doReportDone [%s] finished to %s' % (os.path.basename(self.current_filename), self.remote_address))
        file_sent(self.remote_address, self.current_filename, 'finished', 'udp')
        self.block_size_level += 1
        if self.block_size_level > MAX_BLOCK_SIZE_LEVEL:
            self.block_size_level = MAX_BLOCK_SIZE_LEVEL
        
    def doReportFailed(self):
        # dhnio.Dprint(6, 'transport_udp_session.doReportFailed [%s] failed to %s' % (os.path.basename(self.current_filename), self.remote_address))
        file_sent(self.remote_address, self.current_filename, 'failed', 'udp', None, 'sending failed')
        self.block_size_level -= 2
        if self.block_size_level < 0:
            self.block_size_level = 0

    def doDestroyMe(self):
        global _SessionsDict
        _SessionsDict.pop(self.remote_address)
        automats.get_automats_by_index().pop(self.index)
    
    def sendDatagram(self, data):
        # dhnio.Dprint(10, '    [UDP] %d bytes to %s (%s)' % (len(data), self.remote_address, nameurl.GetName(self.remote_idurl)))
        if self.client is not None:
            try:
                self.client.transport.write(data, self.remote_address)
                return True
            except:
                return False 
        else:
            dhnio.Dprint(2, 'transport_udp_session.sendDatagram WARNING client is None')
            return False      
        
    def incommingFileDone(self):
        file_received(self.filename_in, 'finished', 'udp', self.remote_address)
        os.close(self.fd_in)
        self.fd_in = None
        self.filename_in = ''             
        
    def makeFileID(self):
        return int(str(int(time.time() * 100.0))[4:])   
    
    def addOutboxFile(self, filename):
        self.outbox_queue.append(filename)
        
    def putOutboxFile(self, filename):
        self.outbox_queue.insert(0, filename) 
    
    def buildInboxFile(self, file_id):
        fd, filename = tmpfile.make("udp-in")
        for block_id in range(len(self.incommingFiles[file_id])):
            data = self.incommingFiles[file_id][block_id]
            os.write(fd, data)
        os.close(fd)
        file_received(filename, 'finished', 'udp', self.remote_address)   
    
    def sendBlock(self, block_id):
        global _SendControlFunc
        data = self.outgoingFiles[self.current_file_id][block_id]
        if _SendControlFunc is not None:
            more_bytes = _SendControlFunc(self.last_bytes_sent, len(data))
            if more_bytes < len(data):
                return False
        self.last_bytes_sent = len(data)
        datagram = COMMANDS['DATA']
        datagram += struct.pack('i', self.current_file_id)
        datagram += struct.pack('i', block_id)
        datagram += struct.pack('i', self.current_blocks)
        datagram += struct.pack('i', len(data))
        datagram += data
        return self.sendDatagram(datagram)
#        dhnio.Dprint(8, 'transport_udp_session.sendBlock [%d,%d] (%d bytes) window=%d' % (
#            self.current_file_id, block_id, len(datagram), len(self.sliding_window)))
         
    def markFailedBlock(self, block_id):
        if not self.failedBlocks.has_key(self.current_file_id):
            self.failedBlocks[self.current_file_id] = {}
        if not self.failedBlocks[self.current_file_id].has_key(block_id):
            self.failedBlocks[self.current_file_id][block_id] = 0
        self.failedBlocks[self.current_file_id][block_id] += 1
        if self.failedBlocks[self.current_file_id][block_id] >= BLOCK_RETRIES:
            self.hasFailedBlocks = True
            # dhnio.Dprint(8, 'transport_udp_session.markFailedBlock (%d,%d) is failed to send' % (self.current_file_id, block_id))
            return True
        return False

    def eraseOldFileIDs(self):
        if len(self.receivedFiles) > 10:
            file_ids = self.receivedFiles.keys()
            cur_tm = time.time()
            for file_id in file_ids:
                if cur_tm - self.receivedFiles[file_id] > 60 * 5:
                    del self.receivedFiles[file_id]
            del file_ids 
    
#------------------------------------------------------------------------------ 

def sessions():
    global _SessionsDict
    return _SessionsDict.keys()


def is_session_opened(address, idurl=None):
    global _SessionsDict
    if _SessionsDict.has_key(address):
        return True
    return False
        

def outbox_file(address, idurl, filename, fast=False):
    global _SessionsDict
    s = open_session(address, idurl)
    if fast:
        s.putOutboxFile(filename)
    else:
        s.addOutboxFile(filename)


def big_event(event, arg=None):
    global _SessionsDict
    for address in _SessionsDict.keys():
        A(address, event, arg)


def data_received(datagram, address):
    A(address, 'datagram-received', (datagram, address))


def shutdown_all():
    big_event('shutdown')
    
    
def file_received(filename, status, proto='', host=None, error=None, message=''):
    global _ReceiveStatusCallback
    if _ReceiveStatusCallback is not None:
        _ReceiveStatusCallback(filename, status, proto, host, error, message)
    
    
def file_sent(host, filename, status, proto='', error=None, message=''):
    global _SendStatusCallback
    if _SendStatusCallback is not None:
        _SendStatusCallback(host, filename, status, proto, error, message)


def timer30sec():
    global _Timer30sec
    big_event('timer-30sec')
    _Timer30sec = reactor.callLater(random.randint(25,35), timer30sec)


def timer10sec():
    global _Timer10sec
    big_event('timer-10sec')
    _Timer10sec = reactor.callLater(random.randint(5,15), timer10sec)


def timer1sec():
    global _Timer1sec
    big_event('timer-1sec')
    _Timer1sec = reactor.callLater(1, timer1sec)
    
#------------------------------------------------------------------------------ 
    
def SetReceiveStatusCallback(cb):
    global _ReceiveStatusCallback
    _ReceiveStatusCallback = cb
    
    
def SetSendStatusCallback(cb):
    global _SendStatusCallback
    _SendStatusCallback = cb
    
def SetSendControlFunc(f):
    global _SendControlFunc
    _SendControlFunc = f
    
def SetReceiveControlFunc(f):
    global _ReceiveControlFunc
    _ReceiveControlFunc = f
    
#------------------------------------------------------------------------------ 

    
def StartTimers():
    global _Timer30sec
    global _Timer10sec
    global _Timer1sec
    if _Timer30sec is None:
        timer30sec()
    if _Timer10sec is None: 
        timer10sec()
    if _Timer1sec is None:
        timer1sec()


def StopTimers():
    global _Timer30sec
    global _Timer10sec
    global _Timer1sec
    if _Timer30sec:
        _Timer30sec.cancel()
        _Timer30sec = None
    if _Timer10sec:
        _Timer10sec.cancel()
        _Timer10sec = None
    if _Timer1sec:
        _Timer1sec.cancel()
        _Timer1sec = None








#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#  PREPRO  - this is not our code and we need to replace or just remove
#  This is not so necessary as IP mappings to locations are getting very good.
#  But we would like to know if someone has at least 1 or 2 routers between them
#   and a supplier so there is good chance  they are not just in the same building.

#DHN can ask clients to do traceroutes to others and report the results.
#Results are probably a list of IP addresses to get there.
#With this we can get a good map of the routers/IPs for at least
#as much of the Internet as our customers cover.  This way we can make sure that
#we don't put all of a users backup sights in the same building.
#
#  ping
#  bandwidth - historical or run a test
#  latency   - historical or run a test
#
#
#On Linux, traceroute needs to be done as root.
# There is some python code out there that works on Windows and Linux.
#
#!python
#-----------------------------------------------------------------------------
# Name:        traceroute.py
# Purpose:     Trace route to host
#
# Author:      Joe Brown
#
# Created:     2003/21/03
# RCS-ID:      $Id: traceroute.py,v 1.1 2003/03/21 14:34:28 shmengie Exp $
# Copyright:   (c) 2003
# Licence:     GPL
#-----------------------------------------------------------------------------
import socket, sys, time, os.path
import lib.misc as misc; import lib.dhnio as dhnio

def traceroute(*args):
    argc = len(args[:])
    fqdn = True
    maxHops = 30
    if argc == 1:
        host=args[0]
    elif argc == 2:
        if args[0]=='-d':
            fqdn = False
            host = args[1]
        else:
            usage()
    else:
        usage()
    qport = 65500 # arbitrary port, hopefully not in use
    ttl = 1
    dest_ha = socket.gethostbyname(host)
    if dest_ha != host:
        print "Tracing route to %s (%s)" % (dest_ha, host)
    else:
        print "Tracing route to %s" % dest_ha
    while 1:
        print "%2d" % ttl,
        ha = '*'
        for i in range(3):
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
            if (dhnio.Windows()):
                s.setsockopt(socket.SOL_SOCKET, socket.SO_RCVTIMEO, 300)
            s.setsockopt(socket.SOL_IP, socket.IP_TTL, ttl)

            s2 = socket.socket(socket.AF_INET, socket.SOCK_RAW, socket.IPPROTO_ICMP)
            s2.bind(('',qport))
            if (dhnio.Windows()):
                s2.setsockopt(socket.SOL_SOCKET, socket.SO_RCVTIMEO, 300)
            addr = ""
            start = time.clock()
            s.sendto('', (host,qport))
            try:
                data, addr = s2.recvfrom(512)
                end = time.clock()
                wait = end - start
                ha=addr[0]
                print "%6.1f " % (wait * 1000),
            except:
                data = None
                addr = ("error", "error", "error")
                print "%8s " % '*',
            s.close()
            s2.close()
        print "%-16s" % ha,
        hname = ''
        if ha == '*':
            hname = '*'
        else:
            if fqdn:
                try:
                    hn = socket.gethostbyaddr(ha)
                    hname = hn[0]
                except:
                    pass
                if not hname or hname == ha:
                    hname = ''
        print hname
        ttl+=1
        if ttl > maxHops or ha == dest_ha :
            break

def usage():
    print >> sys.stderr, "%s [-d] target_host" % os.path.basename(sys.argv[0])
    print >> sys.stderr, " -d               No hostname resolution"
    sys.exit(1)
if __name__=='__main__':
    print "Must be run as root on Linux"
    traceroute(*sys.argv[1:])





#!/usr/bin/python
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
# DHNBLOCK.PY
#
# Higher level code interfaces with dhnblock.py so that it does not have to deal
# with ECC stuff.  We write or read a large block at a time (maybe 64 MB say).
# When writing we generate all the ECC information, and when reading we will
# use ECC to recover lost information so user sees whole block still.
#
# We have to go to disk.  The normal mode is probably that there are a few machines that
# are slow and the rest move along.  We want to get the backup secure as soon as possible.
# It can be secure even if 5 to 10 suppliers are not finished yet.  But this could be
# a lot of storage, so we should be using disk.
#
# We want to generate a pool of writes to do, and put in more as it gets below some
# MB limit.  But we should not be limited by a particular nodes speed.
#
# The DHNpacket will have all the info about where it is going etc.
# We number them with our block number and the supplier numbers.
#
# Going to disk should let us do restarts after crashes without much trouble.
#
# Digital Signatures and timestamps are done on dhnblocks.  Signatures are also
# done on dhnpackets.
#
# RAIDMAKE:
#     This object can be asked to generate any/all dhnpacket(s) that would come from this
#     dhnblock.
# RAIDREAD:
#     It can also rebuild the dhnblock from packets and will
#     generate the read requests to get fetch the packets.
#
# import base64
# data64=base64.encodestring(data)    #  in case something can not handle binary remember this
# data=base64.decodestring(data64)    #  in case something can not handle binary remember this
#

import os
import sys
from twisted.internet.defer import Deferred

#if __name__ == '__main__':
#    sys.path.insert(0, os.path.abspath('..'))

import lib.misc as misc
import lib.dhnio as dhnio
import lib.dhncrypto as dhncrypto
import lib.contacts as contacts



#  The only 2 things secret in here will be the EncryptedSessionKey and EncryptedData
#  Scrubbers may combine-packets/unserialize/inspect-blocks/check-signatures
class dhnblock:
    CreatorID=""            # http://cate.com/id1.xml  - so people can check signature - says PK type too
    BackupID=""             # Creator's ID for the backup this packet is part of
    BlockNumber=""          # number of this block
    EncryptedData=""        # data may have some padding so multiple of crypto chunck size
                            # and multiple of #nodes in eccmap (usually 64) for division
                            # into dhnpackets
    Length=0                # real length of data when cleartext (encrypted may be padded)
    LastBlock = str(False)  # should now be "True" or "False" - careful in using
    SessionKeyType=""       # which crypto is used for session key
    EncryptedSessionKey=""  # encrypted with our public key so only we can read this
    Other=""                # could be be for professional timestamp company or other future features
    Signature=""            # Digital signature by Creator - verifiable by public key in creator identity

    def __init__ (self, CreatorID, BackupID, BlockNumber, SessionKey, SessionKeyType, LastBlock, Data,):# SignInThread=False):
        self.CreatorID = CreatorID
        self.BackupID = BackupID
        self.BlockNumber = str(BlockNumber)
        self.EncryptedSessionKey = dhncrypto.EncryptLocalPK(SessionKey)
        self.SessionKeyType = SessionKeyType
        self.Length = str(len(Data))
        self.LastBlock = str(bool(LastBlock))               
        #DataLonger = misc.RoundupString(Data, 24)
        self.EncryptedData = dhncrypto.EncryptWithSessionKey(SessionKey, Data) # DataLonger
        self.Signature = None

        SignInThread = False
        if not SignInThread:
            self.SigningDefer = None
            self.Sign()
        else:
            self.SigningDefer = Deferred()
            self.SignInThread()

    def __repr__(self):
        return 'dhnblock (BackupID=%s BlockNumber=%s Length=%s LastBlock=%s)' % (str(self.BackupID), str(self.BlockNumber), str(self.Length), self.LastBlock)

    def SessionKey(self):
        return(dhncrypto.DecryptLocalPK(self.EncryptedSessionKey))

    def GenerateHashBase(self):
        sep = "::::"
        # PREPRO needs to have all fields and separator
        StringToHash = self.CreatorID + sep + self.BackupID + sep + self.BlockNumber + sep + self.SessionKeyType + sep + self.EncryptedSessionKey + sep + self.Length + sep + self.LastBlock + sep + self.EncryptedData
        return StringToHash

    def GenerateHash(self):
        return dhncrypto.Hash(self.GenerateHashBase())

    def Sign(self):
##        dhnio.Dprint(14, 'dhnblock.Sign ' + str(self))
        self.Signature = self.GenerateSignature()  # usually just done at packet creation
        return self

    def SignInThread(self, SignedCallback=None):
        pass
##        dhnio.Dprint(14, 'dhnblock.SignInThread')
##        self.SigningDefer = threads.deferToThread(self.Sign)
##        if SignedCallback is not None:
##            self.SigningDefer.addCallback(SignedCallback)

##        d = signer.add(self.GenerateHashBase())
##        def done(result):
##            dhnio.Dprint(14, 'dhnpacket.SignInThread.done')
##            self.Signature = result
##            self.SigningDefer.callback(self)
##        def fail(result):
##            dhnio.Dprint(1, 'dhnpacket.SignInThread.fail ERROR. starting self Sign')
##            self.Sign()
##            self.SigningDefer.callback(self)
##        d.addCallback(done)
##        d.addErrback(fail)


    def GenerateSignature(self):
        return dhncrypto.Sign(self.GenerateHash())

    def Ready(self):
        return self.Signature is not None

    def Valid(self):
        if not self.Ready():
            dhnio.Dprint(4, "dhnblock.Valid WARNING block is not ready yet " + str(self))
            return False

        hash = self.GenerateHash()

        ConIdentity = contacts.getContact(misc.getLocalID())
        if ConIdentity is None:
            dhnio.Dprint(2, "dhnblock.Valid WARNING could not get Identity so returning False")
            return False
##        dhnio.Dprint(9, "dhnblock.Valid have ConIdentity")

        result = dhncrypto.Verify(ConIdentity, hash, self.Signature)    # At block level only work on own stuff
        #dhnio.Dprint(10, "dhnblock.Valid result=" + str(result))
        return result

##    def Valid(self):
##        hash=self.GenerateHash()
##        result=dhncrypto.Verify(misc.getLocalID(), hash, self.Signature)    # At block level only work on own stuff
##        dhnio.Dprint(4, "dhnblock.Valid result=" + str(result))
##        return(result)

    def Data(self):
        SessionKey = self.SessionKey()
        ClearLongData = dhncrypto.DecryptWithSessionKey(SessionKey, self.EnctryptedData)
        return ClearLongData[0:self.Length]    # remove padding

    def Serialize(self):
        e = misc.ObjectToString(self)
        return e

def Unserialize(data):
    newobject = misc.StringToObject(data)
    return newobject








#!/usr/bin/python
#restore.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#  At least for now we will work one block at a time, though packets in parallel.
#  We ask transport_control for all the data packets for a block then see if we
#  get them all or need to ask for some parity packets.  We do this till we have
#  gotten a block with the "LastBlock" flag set.  If we have tried several times
#  and not gotten data packets from a supplier we can flag him as suspect-bad
#  and start requesting a parity packet to cover him right away.
#
#  When we are missing a data packet we pick a parity packet where we have all the
#  other data packets for that parity so we can recover the missing data packet.
#  This network cost for this is just as low as if we read the data packet.
#  But most of the time we won't bother reading the parities.  Just uses up bandwidth.
#
#    We don't want to fire someone till
#    after we have finished a restore in case we have other problems and they might come
#    back to life and save us.  However, we might keep packets for a node we plan to fire.
#    This would make replacing them very easy.
#
#  At the "tar" level a user will have choice of full restore (to new empty system probably)
#  or to restore to another location on disk, or to just recover certain files.  However,
#  in this module we have to read block by block all of the blocks.
#
#
# How do we restore if we lost everything?
#     Our ( public/private-key and eccmap) could be:
#                  1)  at DHN  (encrypted with pass phrase)
#                  2)  on USB in safe or deposit box   (encrypted with passphrase or clear text)
#                  3)  with friends or scrubbers (encrypted with passphrase)
# The other thing we need is the backupIDs which we can get from our suppliers with the ListFiles command.
# The ID is something like   F200801161206  or I200801170401 indicating full or incremental.

import os
import sys
import tempfile
import time

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in restore.py')


from twisted.internet.defer import Deferred


import lib.misc as misc
import lib.dhnio as dhnio
##import lib.dhnpacket as dhnpacket
import lib.eccmap as eccmap
import lib.dhncrypto as dhncrypto
#import lib.transport_control as transport_control
import lib.settings as settings
import lib.packetid as packetid
import lib.contacts as contacts
import lib.tmpfile as tmpfile


import raidread
import dhnblock
#import guistatus
import io_throttle
import events


#------------------------------------------------------------------------------ 


class restore:
    def __init__(self, BackupID, OutputFile): # OutputFileName 
        dhnio.Dprint(6, "restore.restore init for backup " + BackupID + ", ecc " + eccmap.CurrentName())
        self.CreatorID = misc.getLocalID()
        self.BackupID = BackupID
        #self.File = open(OutputFileName, "wb")
        self.File = OutputFile
        # is current active block - so when add 1 we get to first, which is 0
        self.BlockNumber = -1              
        self.OnHandData = []
        self.OnHandParity = []
        self.AbortState = False
        self.Done = False
        self.EccMap = eccmap.Current()
        self.LastAction = time.time()
        # Check how things are going every second
        self.PollTime = 1
        # For anyone who wants to know when we finish
        self.MyDeferred = Deferred()       
        self.packetInCallback = None
        
        events.info('restore', 'start restoring backup %s' % self.BackupID)
        
        reactor.callLater(.01, self.StartNewBlock)
        reactor.callLater(self.PollTime, self.MonitorLoop) # Get polling loop going


    def SetPacketInCallback(self, cb):
        self.packetInCallback = cb


    def PacketCameIn(self, NewPacket):
        if self.AbortState:
            return
        if NewPacket is None:
            return
        self.LastAction = time.time()
        #dhnio.Dprint(12, "restore.PacketCameIn seeing BlockNumber %s while on %s " % (NewPacket.BlockNumber(), self.BlockNumber))

        if NewPacket.BlockNumber() != str(self.BlockNumber):
            dhnio.Dprint(6, "restore.PacketCameIn WARNING ignoring old packet "+ str(NewPacket.BlockNumber()) + "-"+ str(NewPacket.SupplierNumber())+ " while on "+ str(self.BlockNumber))
            return

        SupplierNumber = packetid.SupplierNumber(NewPacket.PacketID)

        if NewPacket.DataOrParity() == misc.Data():
            self.OnHandData[SupplierNumber] = True
            fname = self.BackupID + "-" + str(self.BlockNumber) + "-" + str(SupplierNumber) + "-Data"

        elif NewPacket.DataOrParity() == misc.Parity():
            self.OnHandParity[SupplierNumber] = True
            fname = self.BackupID + "-" + str(self.BlockNumber) + "-" + str(SupplierNumber) + "-Parity"

        else:
            dhnio.Dprint(2, "restore.PacketCameIn ERROR %s is not Data or Parity" % NewPacket.BlockNumber())
            self.FailStop()
            return
        
        # filename = os.path.join(tmpfile.subdir('data-par'), fname)
        filename = os.path.join(settings.getLocalBackupsDir(), fname)

        # either way the payload of packet is saved
        dhnio.WriteFile(filename, NewPacket.Payload)                
        dhnio.Dprint(8, "restore.PacketCameIn saved as " + filename)

        if self.EccMap.Fixable(self.OnHandData, self.OnHandParity):
            io_throttle.DeleteBackupRequests(self.BackupID + "-" + str(self.BlockNumber))
            self.PacketsToBlock()

        if self.packetInCallback is not None:
            self.packetInCallback(self.BackupID, NewPacket)


    def PacketsToBlock(self):
        if self.AbortState:
            return
        
        self.LastAction = time.time()
        dhnio.Dprint(6, "restore.PacketsToBlock - starting - block " + str(self.BlockNumber))
        
        io_throttle.DeleteBackupRequests(self.BackupID + "-" + str(self.BlockNumber))
        
        #filename = os.path.join(tmpfile.subdir('data-par'), "newblock-" + self.BackupID)
        filename = os.path.join(settings.getLocalBackupsDir(), "newblock-" + self.BackupID)

        raidread.raidread(filename, eccmap.CurrentName(), self.BackupID, self.BlockNumber)
        blockbits = dhnio.ReadBinaryFile(filename)
        splitindex = blockbits.index(":")
        lengthstring = blockbits[0:splitindex]
        try:
            #dhnio.Dprint(10, "restore.PacketsToBlock - BlockNumber=%s  splitindex=%s lengthstring=%s " % (self.BlockNumber, splitindex, lengthstring))
            datalength = int(lengthstring)                                  # real length before raidmake/ECC
            blockdata = blockbits[splitindex+1:splitindex+1+datalength]     # remove padding from raidmake/ECC
            #dhnio.Dprint(10, "restore.PacketsToBlock - got blockdata len is %s" % len(blockdata))
            newblock = dhnblock.Unserialize(blockdata)                        # convert to object
            #dhnio.Dprint(10, "restore.PacketsToBlock - Unserialize worked ")
        except:
            self.FailStop()
            return
        
        if newblock is None:
            self.FailStop()
            return

        if not newblock.Valid():
            self.FailStop()
            return
        
        dhnio.Dprint(6, "restore.PacketsToBlock " + str(newblock))

        SessionKey = dhncrypto.DecryptLocalPK(newblock.EncryptedSessionKey)
        paddeddata = dhncrypto.DecryptWithSessionKey(SessionKey, newblock.EncryptedData)
        newlen = int(newblock.Length)
        data = paddeddata[:newlen]
        try:
            self.File.write(data)
            # Add to the file where all the data is going
        except:
            pass

        if newblock.LastBlock == str(True):
            dhnio.Dprint(2, "restore.PacketsToBlock has finished EOF block.") 
            reactor.callLater(.01,self.RestoreComplete)
        else:
            reactor.callLater(.01,self.StartNewBlock)


    # PREPRO - right now we fetch all the parities.
    # Could save bandwidth by only fetching needed ones.
    def StartNewBlock(self):
        if self.AbortState:
            return
        self.LastAction = time.time()
        self.BlockNumber += 1                               # current one we are putting together
        dhnio.Dprint(12, "restore.StartNewBlock " + str(self.BlockNumber))
        self.OnHandData=[False]*self.EccMap.datasegments
        self.OnHandParity=[False]*self.EccMap.paritysegments
        self.DoNeededRequests()


    def DoNeededRequests(self):
        packetsToRequest = self.GetPacketNeeds()

        if self.EccMap.Fixable(self.OnHandData, self.OnHandParity):
            dhnio.Dprint(2, "restore.StartNewBlock We have enough data to make block " + str(self.BlockNumber))
            reactor.callLater(.01, self.PacketsToBlock)
            
        else:
            for packetId in packetsToRequest:
                SupplierNumber = packetId[0:packetId.rfind("-")]
                SupplierNumber = SupplierNumber[SupplierNumber.rfind("-")+1]
                io_throttle.QueueRequestFile(
                    self.PacketCameIn, 
                    self.CreatorID, 
                    packetId, 
                    self.CreatorID, 
                    contacts.getSupplierID(SupplierNumber))


    def GetPacketNeeds(self):
        packetsToRequest = []

        for SupplierNumber in range(self.EccMap.datasegments):
            PacketID = packetid.MakePacketID(self.BackupID, self.BlockNumber, misc.Data(), SupplierNumber)
            # if os.path.exists(os.path.join(tmpfile.subdir('data-par'), PacketID)):
            if os.path.exists(os.path.join(settings.getLocalBackupsDir(), PacketID)):
                self.OnHandData[SupplierNumber] = True
            else:
                packetsToRequest.append(PacketID)
                
        for SupplierNumber in range(self.EccMap.paritysegments):
            PacketID = packetid.MakePacketID(self.BackupID, self.BlockNumber, misc.Parity(), SupplierNumber)
            # if os.path.exists(os.path.join(tmpfile.subdir('data-par'), PacketID)):
            if os.path.exists(os.path.join(settings.getLocalBackupsDir(), PacketID)):
                self.OnHandParity[SupplierNumber] = True
            else:
                packetsToRequest.append(PacketID)
                
        return packetsToRequest


#    def CountOnHandPackets(self, onHandList):
#        result=0
#        for eachitem in onHandList:
#            if (eachitem==True):
#                result += 1
#        return(result)


#    #  Could do something more here PREPRO
#    def CheckOnBlock(self):
#        dhnio.Dprint(12, "restore.CheckOnBlock OnHandData=" + str(self.CountOnHandPackets(self.OnHandData)) + "   Parity="+ str(self.CountOnHandPackets(self.OnHandParity)) + " Fixable=" + str(self.EccMap.Fixable(self.OnHandData, self.OnHandParity)))


    def MonitorLoop(self):
        dhnio.Dprint(12, "restore.MonitorLoop")

        if not self.Done:
            if self.AbortState:
                self.CloseWithAbort()
                return
            
            # if we've not done anything in last 60 seconds 
            # (no requests, no incoming, no block rebuilds), 
            # make sure we're on track)
            if time.time() - self.LastAction > 60:
                reactor.callLater(.01, self.DoNeededRequests)
                
            reactor.callLater(self.PollTime, self.MonitorLoop)
            
        else:
            dhnio.Dprint(12, "restore.ReadLoop AtEOF")             # no more CallMeLater


    def FailStop(self):  # when we have an error in the restore, stop
        dhnio.Dprint(1, "restore.FailStop ERROR - new block does not look good")
        self.File.close()
        self.Done = True
        io_throttle.DeleteBackupRequests(self.BackupID + "-")
        self.MyDeferred.errback(self.BackupID+" Bad Block!!!")
        events.notify('restore', 'backup %s: failed to restore block number %d' % (self.BackupID, self.BlockNumber))


    def RestoreComplete(self):
        dhnio.Dprint(2, "restore.RestoreComplete - restore has finished. All is well that ends well.")
        self.File.close()
        self.Done = True
        io_throttle.DeleteBackupRequests(self.BackupID + "-")
        self.MyDeferred.callback(self.BackupID+' done')
        events.info('restore', 'backup %s restored successfully')


    def CloseWithAbort(self):
        dhnio.Dprint(8, "restore.CloseWithAbort")
        self.File.close()
        self.Done = True
        io_throttle.DeleteBackupRequests(self.BackupID + "-")
        self.MyDeferred.callback(self.BackupID+' aborted')   # succeed
        events.info('restore', 'restoring of backup %s were aborted')


    def Abort(self): # for when user clicks the Abort restore button on the gui
        dhnio.Dprint(8, "restore.Abort")
        self.AbortState = True




#!/usr/bin/env python
#installer.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in installer.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall


import lib.dhnio as dhnio
import lib.dhnnet as dhnnet
import lib.misc as misc
import lib.nameurl as nameurl
import lib.packetid as packetid
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control
import lib.diskusage as diskusage
from lib.automat import Automat


import initializer
import p2p_connector
import identity_registrator
import identity_restorer
import lib.automats as automats

import dhninit
import dhnupdate
import identitypropagate
import install
import webcontrol


_Installer = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _Installer
    if _Installer is None:
        _Installer = Installer('installer', 'AT_STARTUP', 4)
    if event is not None:
        _Installer.automat(event, arg)
    return _Installer


class Installer(Automat):
    output = {}
    MESSAGE_01 = ('incorrect user name', 'red')
    RECOVER_RESULTS = {
        'remote_identity_not_valid':  ('remote Identity is not valid', 'red'),
        'invalid_identity_source':    ('incorrect source of the Identity file', 'red'),
        'invalid_identity_url':       ('incorrect Identity file location', 'red'),
        'remote_identity_bad_format': ('incorrect format of the Identity file', 'red'),
        'incorrect_key':              ('Private Key is not valid', 'red'),
        'idurl_not_exist':            ('Identity URL address not exist or not reachable at this moment', 'blue'),
        'signing_error':              ('unable to sign the local Identity file', 'red'),
        'signature_not_match':        ('remote Identity and Private Key did not match', 'red'),
        'central_failed':             ('unable to connect to the Central server, try again later', 'blue'),
        'success':                    ('account restored!', 'green'), }

    def getOutput(self, state=None):
        if state is None:
            state = self.state
        return self.output.get(state, {})

    def init(self):
        self.flagCmdLine = False

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('INSTALL ' + newstate)
        initializer.A('installer.state', newstate)

    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'init':
                self.flagCmdLine = False
                self.state = 'WHAT_TO_DO?'
            elif event == 'register-cmd-line':
                self.flagCmdLine = True
                identity_registrator.A('start', arg)
                self.state = 'REGISTER'
            elif event == 'recover-cmd-line':
                self.flagCmdLine = True
                identity_restorer.A('start', arg)
                self.state = 'RECOVER'
        #---WHAT_TO_DO?---
        elif self.state == 'WHAT_TO_DO?':
            if event == 'register-selected':
                self.doUpdate()
                self.state = 'INPUT_NAME'
            elif event == 'recover-selected':
                self.doUpdate()
                self.state = 'LOAD_KEY'
        #---INPUT_NAME---
        elif self.state == 'INPUT_NAME':
            if event == 'back':
                self.doUpdate()
                self.state = 'WHAT_TO_DO?'
            elif event == 'register-start' and self.isNameValid(arg):
                self.doClearOutput()
                identity_registrator.A('start', arg)
                self.doUpdate()
                self.state = 'REGISTER'
            elif event == 'register-start' and not self.isNameValid(arg):
                self.doPrint(self.MESSAGE_01)
                self.doUpdate()
            elif event == 'print':
                self.doPrint(arg)
                self.doUpdate()
        #---LOAD_KEY---
        elif self.state == 'LOAD_KEY':
            if event == 'back':
                self.doUpdate()
                self.state = 'WHAT_TO_DO?'
            elif event == 'load-from-file':
                self.doReadKey(arg)
                self.doUpdate()
            elif event == 'paste-from-clipboard':
                self.doPasteKey()
                self.doUpdate()
            elif event == 'restore-start':
                self.doClearOutput()
                identity_restorer.A('start', arg)
                self.doUpdate()
                self.state = 'RECOVER'
            elif event == 'print':
                self.doPrint(arg)
                self.doUpdate()
        #---REGISTER---
        elif self.state == 'REGISTER':
            if event == 'identity_registrator.state' and arg == 'REGISTERED' and not self.flagCmdLine:
                self.doUpdate()
                self.state = 'AUTHORIZED'
            elif event == 'identity_registrator.state' and arg == 'READY' and not self.flagCmdLine:
                self.doUpdate()
                self.state = 'INPUT_NAME'
            elif event == 'print':
                self.doPrint(arg)
                self.doUpdate()
            elif event == 'identity_registrator.state' and arg in [ 'READY' , 'REGISTERED' ] and self.flagCmdLine:
                self.state = 'DONE'
        #---AUTHORIZED---
        elif self.state == 'AUTHORIZED':
            if event == 'next':
                self.doUpdate()
                self.state = 'CENTRAL'
        #---CENTRAL---
        elif self.state == 'CENTRAL':
            if event == 'central-ready' and self.isCentralValid(arg) and not self.isDirSelected(arg):
                self.doCentralSave(arg)
                self.doUpdate()
                self.state = 'CONTACTS'
            elif event == 'central-ready' and self.isDirSelected(arg):
                self.doDirSave(arg)
                self.doUpdate()
            elif event == 'central-ready' and not self.isCentralValid(arg):
                self.doPrintCentralError(arg)
                self.doUpdate()
        #---CONTACTS---
        elif self.state == 'CONTACTS':
            if event == 'contacts-ready' and self.isContactsValid(arg):
                self.doContactsSave(arg)
                self.doUpdate()
                self.state = 'UPDATES'
            elif event == 'contacts-ready' and not self.isContactsValid(arg):
                self.doPrintContactsError(arg)
                self.doUpdate()
            elif event == 'back':
                self.doUpdate()
                self.state == 'CENTRAL'
        #---UPDATES---
        elif self.state == 'UPDATES':
            if event == 'updates-ready':
                self.doUpdatesSave(arg)
                self.doUpdate()
                self.state = 'DONE'
            elif event == 'back':
                self.doUpdate()
                self.state = 'CONTACTS'
        #---RECOVER---
        elif self.state == 'RECOVER':
            if event == 'identity_restorer.state' and arg == 'DONE' or ( event == 'identity_restorer.state' and arg == 'READY' and self.flagCmdLine ):
                self.doUpdate()
                self.state = 'DONE'
            elif event == 'identity_restorer.state' and arg == 'READY' and not self.flagCmdLine:
                self.state = 'LOAD_KEY'
            elif event == 'print':
                self.doPrint(arg)
                self.doUpdate()
        #---DONE---
        elif self.state == 'DONE':
            pass

    def isNameValid(self, arg):
        if not misc.ValidUserName(arg):
            return False
        return True

    def isDirSelected(self, arg):
        return arg.get('opendir', '') != '' 

    def isCentralValid(self, arg):
        needed = arg.get('needed', '')
        donated = arg.get('donated', '')
        customersdir = arg.get('customersdir', '')
        try:
            neededV = float(needed)
            donatedV = float(donated)
        except:
            return False
        if donatedV < settings.MinimumDonatedMb():
            return False
        if not os.path.isdir(customersdir):
            return False
        if not os.access(customersdir, os.W_OK):
            return False
        freeBytes, totalBytes = diskusage.GetDriveSpace(customersdir)
        if freeBytes <= donatedV * 1024 * 1024:
            return False
        return True

    def isContactsValid(self, arg):
        email = arg.get('email', '').strip()
        phone = arg.get('phone', '')
        fax = arg.get('fax', '')
        text = arg.get('text', '')
        if email == '':
            return False
        if not misc.ValidEmail(email):
            return False
        if phone != '' and not misc.ValidPhone(phone):
            return False
        if fax != '' and not misc.ValidPhone(fax):
            return False
        if len(text) > 500:
            return False
        return True

    def isRecoverSuccess(self, arg):
        return arg == 'success'

    def doClearOutput(self):
        for state in self.output.keys():
            self.output[state] = {'data': [('', 'black')]}

    def doPrint(self, arg):
        text, color = arg
        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        self.output[self.state]['data'].append((text, color))
        dhnio.Dprint(0, '  [%s]' % text)

    def doUpdate(self):
        webcontrol.OnUpdateInstallPage()

    def doPrintCentralError(self, arg):
        needed = arg.get('needed', '')
        donated = arg.get('donated', '')
        customersdir = arg.get('customersdir', '')
        try:
            neededV = float(needed)
        except:
            self.doPrint(('incorrect needed space value', 'red'))
            return 
        try:
            donatedV = float(donated)
        except:
            self.doPrint(('incorrect donated space value', 'red'))
            return
        if donatedV < settings.MinimumDonatedMb():
            self.doPrint(('you must donate at least %d MB' % settings.MinimumDonatedMb(), 'blue'))
            return 
        if not os.path.isdir(customersdir):
            self.doPrint(('directory %s not exist' % customersdir, 'red'))
            return 
        if not os.access(customersdir, os.W_OK):
            self.doPrint(('specified folder does not have write permissions', 'red'))
            return 
        freeBytes, totalBytes = diskusage.GetDriveSpace(customersdir)
        if freeBytes <= donatedV * 1024 * 1024:
            self.doPrint(('you do not have enough free space on the hard disk', 'red'))
            return 

    def doCentralSave(self, arg):
        needed = arg.get('needed', '')
        donated = arg.get('donated', '')
        customersdir = arg.get('customersdir', '')
        settings.uconfig().set('central-settings.needed-megabytes', needed+'MB')
        settings.uconfig().set('central-settings.shared-megabytes', donated+'MB')
        settings.uconfig().set('folder.folder-customers', customersdir)
        settings.uconfig().update()

    def doDirSave(self, arg):
        settings.uconfig().set('folder.folder-customers', arg['customersdir'])
        settings.uconfig().update()

    def doPrintContactsError(self, arg):
        email = arg.get('email', '').strip()
        phone = arg.get('phone', '')
        fax = arg.get('fax', '')
        text = arg.get('text', '')
        if email == '':
            self.doPrint(('''<div align=left>
Please enter your e-mail address. 
DataHaven.NET guarantee no spam!
Your e-mail will not be published!. 
We only need to notify you
in case of critical errors
during testing period.</div>
'''.replace('\n', '<br>\n'), 'blue'))
            return 
        if not misc.ValidEmail(email):
            self.doPrint(('incorrect email address', 'red'))
            return 
        if phone != '' and not misc.ValidPhone(phone):
            self.doPrint(('incorrect phone number', 'red'))
            return 
        if fax != '' and not misc.ValidPhone(fax):
            self.doPrint(('incorrect fax number', 'red'))
            return 
        if len(text) > 500:
            self.doPrint(('the text is too long', 'red'))
            return 

    def doContactsSave(self, arg):
        email = arg.get('email', '').strip()
        phone = arg.get('phone', '')
        fax = arg.get('fax', '')
        text = arg.get('text', '')
        settings.uconfig().set('emergency.emergency-email', email)
        settings.uconfig().set('emergency.emergency-phone', phone)
        settings.uconfig().set('emergency.emergency-fax', fax)
        settings.uconfig().set('emergency.emergency-text', text)
        settings.uconfig().update()

    def doUpdatesSave(self, arg):
        shedule = dhnupdate.blank_shedule(arg)
        settings.uconfig().set('updates.updates-shedule', dhnupdate.shedule_to_string(shedule))
        settings.uconfig().update()
    
    def doReadKey(self, arg):
        dhnio.Dprint(2, 'installer.doReadKey arg=[%s]' % str(arg))
        src = dhnio.ReadBinaryFile(arg)
        if len(src) > 1024*10:
            self.doPrint(('file is too big for private key', 'red'))
            return

        try:
            lines = src.split('\n')
            idurl = lines[0]
            keysrc = '\n'.join(lines[1:])
            if idurl != nameurl.FilenameUrl(nameurl.UrlFilename(idurl)):
                idurl = ''
                keysrc = src
        except:
            dhnio.DprintException()
            idurl = ''
            keysrc = src

#        try:
#            lines = src.split('\n')
#            idurl = lines[0]
#            if idurl.find('BEGIN RSA PRIVATE KEY') > -1:
#                keysrc = src
#            if idurl.startswith('http://') or :
#                keysrc = '\n'.join(lines[1:])
#            else:
#                keysrc = src
#                idurl = ''
#            if idurl != nameurl.FilenameUrl(nameurl.UrlFilename(idurl)):
#                idurl = ''
#                keysrc = src
#        except:
#            dhnio.DprintException()
#            idurl = ''
#            keysrc = src

        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        self.output[self.state]['idurl'] = idurl
        self.output[self.state]['keysrc'] = keysrc
        
    def doPasteKey(self):
        src = misc.getClipboardText()
        try:
            lines = src.split('\n')
            idurl = lines[0]
            keysrc = '\n'.join(lines[1:])
            if idurl != nameurl.FilenameUrl(nameurl.UrlFilename(idurl)):
                idurl = ''
                keysrc = src
        except:
            dhnio.DprintException()
            idurl = ''
            keysrc = src
        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        self.output[self.state]['idurl'] = idurl
        self.output[self.state]['keysrc'] = keysrc





import sys
if len(sys.argv) < 4:
    print 'setoption.py <base dir path> <full option name> <value>'
    sys.exit(1)

baseDir = sys.argv[1]
key = sys.argv[2]
value = ' '.join(sys.argv[3:])

import settings
settings.init(baseDir)
settings.uconfig().set(key, value)
settings.uconfig().update()




from __future__ import nested_scopes

from twisted.internet import defer
import sys

class _DeferredCache:
    """ Wraps a call that returns a deferred in a cache. Any subsequent
        calls with the same argument will wait for the first call to
        finish and return the same result (or errback).
    """

    hashableArgs = False
    inProgressOnly = True

    def __init__(self, op, hashableArgs=None, inProgressOnly=None):
        self.op = op
        self.cache = {}
        if hashableArgs is not None:
            self.hashableArgs = hashableArgs
        if inProgressOnly is not None:
            self.inProgressOnly = inProgressOnly

    def cb_triggerUserCallback(self, res, deferred):
        #print "triggering", deferred
        deferred.callback(res)
        return res

    def cb_triggerUserErrback(self, failure, deferred):
        deferred.errback(failure)
        return failure

    def _genCache(self, args, kwargs):
        # This could be better, probably
        try:
            arghash = hash(args)
        except TypeError:
            return None
        kwit = kwargs.items()
        kwit.sort()
        try:
            kwhash = hash(tuple(kwit))
        except TypeError:
            return None
        return (arghash, kwhash)

    def _removeCacheVal(self, res, cacheVal):
        del self.cache[cacheVal]
        return res

    def clearCache(self):
        self.cache = {}

    def call(self, *args, **kwargs):
        # Currently not in progress - start it
        #print "called with", args
        cacheVal = self._genCache(args, kwargs)
        if cacheVal is None and self.hashableArgs:
            raise TypeError('DeferredCache(%s) arguments must be hashable'%(
                                self.op.func_name))

        opdef = self.cache.get(cacheVal)
        if not opdef:
            # XXX assert that it returns a deferred?
            opdef = self.op(*args, **kwargs)
            if cacheVal is not None:
                self.cache[cacheVal] = opdef
            if self.inProgressOnly and cacheVal:
                opdef.addCallbacks(lambda x: self._removeCacheVal(x, cacheVal),
                                   lambda x: self._removeCacheVal(x, cacheVal))

        userdef = defer.Deferred()
        opdef.addCallbacks(lambda x: self.cb_triggerUserCallback(x, userdef),
                           lambda x: self.cb_triggerUserErrback(x, userdef))
        return userdef


def DeferredCache(op=None, hashableArgs=None, inProgressOnly=None):
    """ Use this as a decorator for a function or method that returns a
        deferred. Any subsequent calls using the same arguments will
        be all triggered off the original deferred, all returning the
        same result.
    """
    if op is None:
        return lambda x: DeferredCache(x, hashableArgs, inProgressOnly)
    c = _DeferredCache(op, hashableArgs, inProgressOnly)
    def func(*args, **kwargs):
        return c.call(*args, **kwargs)
    if sys.version_info > (2,4):
        func.func_name = op.func_name
    func.clearCache = c.clearCache
    func.cache_hashableArgs = c.hashableArgs
    func.cache_inProgressOnly = c.inProgressOnly
    return func



#!/usr/bin/python
#backup.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#  This interfaces between a pipe from something like tar and the twisted code
#    for rest of DataHaven.NET
#  We see how many packets are waiting to be sent,
#    and if it is not too many, and we can make more, we make some more.
#
#  Main idea:
#     1) When a backup is started a backup object is created
#     2) We get a file descriptor for the process creating the tar archive
#     3) We always use select/poll before reading so we never block
#     4) We also poll to see if more needed.
#     5) We number/name blocks so can be sure what is what when we read back later
#     6) We call raidmake to split block and make parities
#     7) We put parts into dhnpackets and give these to transport_control


import os
import sys
import time
import cStringIO
import gc


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backup.py')

from twisted.internet import threads
from twisted.internet.defer import Deferred, maybeDeferred


import lib.dhnio as dhnio
import lib.misc as misc
import lib.dhnpacket as dhnpacket
import lib.contacts as contacts
import lib.commands as commands
import lib.settings as settings
import lib.packetid as packetid
import lib.nonblocking as nonblocking
import lib.eccmap as eccmap
import lib.dhncrypto as dhncrypto
import lib.tmpfile as tmpfile
from lib.automat import Automat


import lib.automats as automats
import fire_hire
import contacts_status

import raidmake
import dhnblock
import io_throttle
import events
import backup_db


#-------------------------------------------------------------------------------

class backup(Automat):
    timers = {'timer-01sec':    (0.1, ['RUN', 'READ']), 
              'timer-1sec':     (1, ['SENDING'])}
    
    def __init__(self, backupID, pipe, blockSize=None, resultDeferred=None):
        self.backupID = backupID
        self.eccmap = eccmap.Current()
        self.pipe = pipe
        self.blockSize = blockSize
        if self.blockSize is None:
            self.blockSize = self.eccmap.nodes() * settings.PacketSizeTarget()
        self.ask4abort = False
        self.stateEOF = False
        self.stateReading = False
        #self.currentBlockData = ''
        self.currentBlockData = cStringIO.StringIO()
        self.currentBlockSize = 0
        self.blockNumber = 0
        self.dataSent = 0
        self.blocksSent = 0
        self.closed = False
        self.sendingDelay = 0.1
        self.result = ''
        if resultDeferred is None:
            self.resultDefer = Deferred()
        else:
            self.resultDefer = resultDeferred
        self.packetResultCallback = None
        # need to give time for supplier to receive the packet
        # but do not wait too long because other suppliers receive same data faster
        # this is a number of seconds we give to supplier to receive our packet
        self.PacketFailedTimeout = int(settings.PacketSizeTarget() / settings.SendingSpeedLimit())
        # (packetID, filename) queue for each supplier by his number
        # [suplierNumber](packetID, filename)
        self.outstandingPackets = {}
        # (packetID, filename) that is currently sending for every supplier number
        self.currentlySendingPackets = {} 
        # time when we start sending the packet for each supplier by his number    
        self.sendingBeginTimers = {}
        # how many packets was failed to send for each supplier by his number
        self.sentFailedCounts = {} 
        # failed and success packets for each block by its number
        # store both data and parity packets in this form
        # [blockNumber]['D','P'][success, failed]      
        self.blocksResultDict = {}
        # timed out packets list for each supplier by his number
        self.timedOutPackets = {}
        # (total bytes sent, total duration sending) to each supplier by his number
        # we count only time "during sending packet",
        # this starts from moment of start sending single packet 
        # and moment when we receive an ack on this packet 
        self.sendingStats = {}
        Automat.__init__(self, 'backup', 'AT_STARTUP', 10)
        self.automat('init')
        events.info('backup', 'backup %s started' % self.backupID)
        dhnio.Dprint(6, 'backup.__init__ %s %s %d %d' % (self.backupID, self.eccmap, self.blockSize, self.PacketFailedTimeout))
        
    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'init':
                self.doStartProcessOutgoingFiles()
                self.state = 'RUN'
        #---RUN---
        elif self.state == 'RUN':
            if event == 'timer-01sec' and self.isAborted():
                self.doClose()
                self.doReport()
                self.doDestroyMe()
                self.state = 'ABORTED'
            elif event == 'timer-01sec' and not self.isAborted():
                self.state = 'READ'
        #---READ---
        elif self.state == 'READ':
            if event == 'timer-01sec' and self.isPipeReady() and not self.isEOF() and not self.isReadingNow() and not self.isBlockReady():
                self.doRead()
            elif event == 'timer-01sec' and not self.isReadingNow() and (self.isBlockReady() or self.isEOF()):
                self.doBlock()
                self.state = 'BLOCK'
        #---BLOCK---
        elif self.state == 'BLOCK':
            if event == 'block-ready':
                self.doRaid(arg)
                self.state = 'RAID'
        #---RAID---
        elif self.state == 'RAID':
            if event == 'raid-done' and not self.isEOF():
                self.doSend(arg)
                self.doNewBlock()
                self.state = 'RUN'
            elif event == 'raid-done' and self.isEOF():
                self.doSend(arg)
                self.state = 'SENDING'
        #---SENDING---
        elif self.state == 'SENDING':
            if event == 'timer-1sec' and self.isSentAll():
                self.doClose()
                self.doReport()
                self.doDestroyMe()
                self.state = 'DONE'
            elif event == 'timer-1sec' and self.isAborted():
                self.doClose()
                self.doReport()
                self.doDestroyMe()
                self.state = 'ABORTED'
        #---DONE---
        elif self.state == 'DONE':
            pass
        #---ABORTED---
        elif self.state == 'ABORTED':
            pass

    def isAborted(self):
        return self.ask4abort
         
    def isSentAll(self):
        return len(self.blocksResultDict) == 0 and self.dataSent > 0
    
    def isPipeReady(self):
        return self.pipe is not None and self.pipe.state() in [nonblocking.PIPE_CLOSED, nonblocking.PIPE_READY2READ]
    
    def isBlockReady(self):
        return self.currentBlockSize >= self.blockSize
    
    def isEOF(self):
        return self.stateEOF
    
    def isReadingNow(self):
        return self.stateReading

    def doClose(self):
        self.closed = True
        
    def doDestroyMe(self):
        self.currentBlockData.close()
        del self.currentBlockData
        automats.get_automats_by_index().pop(self.index)
        reactor.callLater(0, backup_db.RemoveRunningBackupObject, self.backupID)
        collected = gc.collect()
        dhnio.Dprint(6, 'backup.doDestroyMe collected %d objects' % collected)

    def doReport(self):
        if self.result == '':
            self.resultDefer.callback(self.backupID)
            events.info('backup', 'backup %s done successfully' % self.backupID)
        elif self.result == 'abort':  
            self.resultDefer.callback(self.backupID+' abort')
            events.info('backup', 'backup %s was aborted' % self.backupID)
        else:
            self.resultDefer.errback(self.backupID)
            dhnio.Dprint(1, 'backup.doReport ERROR %s result is [%s]' % (self.backupID, self.result))
            events.info('backup', 'backup %s failed: %s' % (self.backupID, self.result))

    def doRead(self):
        def readChunk():
            size = self.blockSize - self.currentBlockSize
            if size < 0:
                dhnio.Dprint(1, "backup.readChunk ERROR eccmap.nodes=" + str(self.eccmap.nodes()))
                dhnio.Dprint(1, "backup.readChunk ERROR blockSize=" + str(self.blockSize))
                dhnio.Dprint(1, "backup.readChunk ERROR currentBlockSize=" + str(self.currentBlockSize))
                raise Exception('size < 0, blockSize=%s, currentBlockSize=%s' % (self.blockSize, self.currentBlockSize))
                return ''
            elif size == 0:
                return ''
            if self.pipe is None:
                raise Exception('backup.pipe is None')
                return ''
            if self.pipe.state() == nonblocking.PIPE_CLOSED:
                dhnio.Dprint(4, 'backup.readChunk the state is PIPE_CLOSED !!!!!!!!!!!!!!!!!!!!!!!!')
                return ''
            if self.pipe.state() == nonblocking.PIPE_READY2READ:
                newchunk = self.pipe.recv(size)
                if newchunk == '':
                    dhnio.Dprint(4, 'backup.readChunk pipe.recv() returned empty string')
                return newchunk
            dhnio.Dprint(1, "backup.readChunk ERROR pipe.state=" + str(self.pipe.state()))
            raise Exception('backup.pipe.state is ' + str(self.pipe.state()))
            return ''
        def readDone(data):
            #self.currentBlockData += data
            self.currentBlockData.write(data)
            self.currentBlockSize += len(data)
            self.stateReading = False
            if data == '':
                self.stateEOF = True
            #dhnio.Dprint(12, 'backup.readDone %d bytes' % len(data))
        self.stateReading = True
        maybeDeferred(readChunk).addCallback(readDone)

    def doBlock(self):
        def _doBlock():
            dhnio.Dprint(12, 'backup.doBlock blockNumber=%d size=%d atEOF=%s' % (self.blockNumber, self.currentBlockSize, self.stateEOF))
            #src = self.currentBlockData
            src = self.currentBlockData.getvalue()
            #self.currentBlockData.close()
            #del self.currentBlockData
            ret = dhnblock.dhnblock(
                misc.getLocalID(),
                self.backupID,
                self.blockNumber,
                dhncrypto.NewSessionKey(),
                dhncrypto.SessionKeyType(),
                self.stateEOF,
                src,)
            del src
            return ret
        maybeDeferred(_doBlock).addCallback(
            lambda block: self.automat('block-ready', block),)

    def doRaid(self, arg):
        newblock = arg
        dhnio.Dprint(8, 'backup.doRaid block=%d size=%d eof=%s ' % (
            self.blockNumber, self.currentBlockSize, str(self.stateEOF),))
        fileno, filename = tmpfile.make('backup')
        serializedblock = newblock.Serialize()
        blocklen = len(serializedblock)
        os.write(fileno, str(blocklen) + ":" + serializedblock)
        os.close(fileno)
        threads.deferToThread(raidmake.raidmake, 
                              filename, 
                              self.eccmap.name, 
                              self.backupID, 
                              self.blockNumber).addBoth(
                                  lambda outDir: self.automat('raid-done', newblock))
        del serializedblock

    def doSend(self, arg):
        newblock = arg
        for supplierNum in range(self.eccmap.nodes()):
            # we do all data then all parity
            for DataOrParity in ('Data','Parity'):
                if not self.outstandingPackets.has_key(supplierNum):
                    self.outstandingPackets[supplierNum] = []
                packetId = newblock.BackupID + '-' + str(newblock.BlockNumber) + '-' + str(supplierNum) + '-' + DataOrParity
                # fullfilename = os.path.join(tmpfile.subdir('data-par'), packetId)
                fullfilename = os.path.join(settings.getLocalBackupsDir(), packetId)
                self.outstandingPackets[supplierNum].append((packetId, fullfilename))
        # self.dataSent += len(self.currentBlockData)
        self.dataSent += self.currentBlockSize
        del newblock

    def doNewBlock(self):
        #self.currentBlockData = ''
        self.currentBlockData.close()
        del self.currentBlockData
        self.currentBlockData = cStringIO.StringIO()
        self.currentBlockSize = 0
        self.blockNumber += 1
        if self.packetResultCallback is not None:
            self.packetResultCallback(self.backupID, None)
        
    def doStartProcessOutgoingFiles(self):
        reactor.callLater(0, self.processSending)     
        
    #------------------------------------------------------------------------------ 

    def processSending(self):
        if self.closed:
            return
        if len(self.outstandingPackets) > 0:
            self.sendingDelay = 0.1
        else:
            if self.sendingDelay < 4.0:
                self.sendingDelay *= 2.0
        # let's check all outstanding packets for all suppliers
        for supplierNum in self.outstandingPackets.keys():
            # if nothing to send for this supplier - skip
            if len(self.outstandingPackets[supplierNum]) == 0:
                continue
            # if we already sending a file to this man
            # need to check how long we are doing this
            beginSendingTime = self.sendingBeginTimers.get(supplierNum, None)
            if beginSendingTime is not None:
                if time.time() - beginSendingTime < self.PacketFailedTimeout:
                    # currently sending to the supplier - check next supplier
                    continue
                dhnio.Dprint(6, 'backup.processSending timeout sending for supplier %d' % supplierNum)
                # so this packet is timed out
                # and we increase failed counter for this man
                if not self.sentFailedCounts.has_key(supplierNum):
                    self.sentFailedCounts[supplierNum] = 0
                self.sentFailedCounts[supplierNum] += 1
                # remove this packet from currently sending packets
                timedOutPacketID, filename = self.currentlySendingPackets.pop(supplierNum) 
                # now we want to mark this packet as timed out
                # so when packetAcked or packetFailed will be fired by io_throttle
                # we will check and do not update block result 
                if not self.timedOutPackets.has_key(supplierNum):
                    self.timedOutPackets[supplierNum] = []
                self.timedOutPackets[supplierNum].append(timedOutPacketID)
                # remove this timer because we know now that packet is failed
                self.sendingBeginTimers.pop(supplierNum)
                # finally update block result 
                self.blockResult(timedOutPacketID, False)
            # now we know we did not sending to this supplier
            # and there are some files to be sent to him, take one from his queue
            packetId, fullfilename = self.outstandingPackets[supplierNum].pop(0)
            # if we already have 2 failed packets sent to this man
            # we do not want to send any more, because no trust to him
            # backup_monitor will reconstruct this backup later
            if self.sentFailedCounts.has_key(supplierNum):
                #dhnio.Dprint(6, 'backup.processSending supplier %d failed counts is %d' % (supplierNum, self.sentFailedCounts[supplierNum]))
                if self.sentFailedCounts[supplierNum] >= 2:
                    self.blockResult(packetId, False)
                    continue
            # do sending  
            io_throttle.QueueSendFile(
                fullfilename,
                packetId,
                contacts.getSupplierID(supplierNum),
                misc.getLocalID(),
                self.packetAcked,
                self.packetFailed,)
            # save current packetId and filename
            self.currentlySendingPackets[supplierNum] = (packetId, fullfilename) 
            # remember current time for this supplier
            self.sendingBeginTimers[supplierNum] = time.time()
            #dhnio.Dprint(6, 'backup.processSending begin sending [%s] to supplier %d' % (packetId, supplierNum))
        # call again later
        # attenuation
        reactor.callLater(self.sendingDelay, self.processSending)


    def packetAcked(self, packet, ownerID, packetID):
        if self.closed:
            return
        # take supplier number ...
        supplierNum = int(packetID.split('-')[2])
        # if packet is timed out - skip
        if self.timedOutPackets.has_key(supplierNum):
            if packetID in self.timedOutPackets[supplierNum]:
                # but remove it from this list to be smart
                self.timedOutPackets[supplierNum].remove(packetID)
                return
        # ... and remove from currently sending packets
        currentPacketID, filename = self.currentlySendingPackets.pop(supplierNum)
        if packetID != currentPacketID:
            raise Exception('incorrect packetID=%s for supplier %d, currentlySendingPackets=%s' % (packetID, supplierNum, self.currentlySendingPackets))
        # and also remove timer so we know that supplier is free and able to receive next packet
        startTimePacket = self.sendingBeginTimers.pop(supplierNum)
        # increase amount of total bytes sent to this man
        # supplier puts number of received bytes in the Ack
        # and we know how much he received in last packet of Data     
        if not self.sendingStats.has_key(supplierNum):
            self.sendingStats[supplierNum] = [0, 0]
        self.sendingStats[supplierNum][0] += int(packet.Payload)
        # remember the period of time how long we sent this packet
        self.sendingStats[supplierNum][1] += time.time() - startTimePacket
        # now we can remove the local file because remote supplier got it and ...
        if settings.getGeneralLocalBackups() is False:
            # ... user do not want to keep local backups
            if settings.getGeneralWaitSuppliers() is True:
                # but he want to be sure - all suppliers are green for long time
                if time.time() - fire_hire.GetLastFireTime() > 24*60*60:
                    # that is fine - we know those people for long. they are trusted.
                    # now we do not need local file
                    # fullfilename = os.path.join(tmpfile.subdir('data-par'), packetID)
                    fullfilename = os.path.join(settings.getLocalBackupsDir(), packetID)
                    if os.path.isfile(fullfilename):
                        dhnio.Dprint(2, 'backup.packetAcked will remove [%s]' % fullfilename)
                        try:
                            os.remove(fullfilename)
                        except:
                            dhnio.DprintException()
        # update block result finally
        self.blockResult(packetID, True)
        # update GUI
        if self.packetResultCallback is not None:
            self.packetResultCallback(self.backupID, packet)
        #dhnio.Dprsint(10, 'backup.packetAcked [%s] outstanding=%d' % (packetID, len(self.outstanding)))


    def packetFailed(self, creatorID, packetID, why):
        if self.closed:
            return
        # take supplier number ...
        supplierNum = int(packetID.split('-')[2])
        #dhnio.Dprint(6, 'backup.packetFailed [%s] for supplier %d' % (packetID, supplierNum))
        # if packet is timed out - skip
        if self.timedOutPackets.has_key(supplierNum):
            if packetID in self.timedOutPackets[supplierNum]:
                # but remove it from this list to be smart
                self.timedOutPackets[supplierNum].remove(packetID)
                return
        # ... and remove from currently sending packets
        currentPacketID, filename = self.currentlySendingPackets.pop(supplierNum)
        if packetID != currentPacketID:
            raise Exception('incorrect packetID=%s for supplier %d, currentlySendingPackets=%s' % (packetID, supplierNum, self.currentlySendingPackets))
        # and also remove timer so we know that supplier is free and able to receive next packet
        startTime = self.sendingBeginTimers.pop(supplierNum)
        # increase failed counter
        if not self.sentFailedCounts.has_key(supplierNum):
            self.sentFailedCounts[supplierNum] = 0
        self.sentFailedCounts[supplierNum] += 1
        # update block result finally
        self.blockResult(packetID, False)
        # do notify gui
        if self.packetResultCallback is not None:
            self.packetResultCallback(self.backupID, None)


    def blockResult(self, PacketID, Result):
        backupID, blockNum, supplierNum, dataORparity  = PacketID.split('-')
        blockNum = int(blockNum)
        supplierNum = int(supplierNum)
        DorP = dataORparity[0] # "D" or "P"
        if not self.blocksResultDict.has_key(blockNum):
            self.blocksResultDict[blockNum] = {'D': [0, 0], 'P': [0, 0]}
        self.blocksResultDict[blockNum][DorP][int(Result)] += 1
        succesData = self.blocksResultDict[blockNum]['D'][1]
        failedData = self.blocksResultDict[blockNum]['D'][0]
        succesParity = self.blocksResultDict[blockNum]['P'][1]
        failedParity = self.blocksResultDict[blockNum]['P'][0]
        if succesData + failedData >= self.eccmap.nodes() and succesParity + failedParity >= self.eccmap.nodes():
            self.blocksResultDict.pop(blockNum)
            self.blocksSent += 1
            if succesData < self.eccmap.DataNeeded():
                dhnio.Dprint(4, 'backup.blockResult WARNING block %d-Data keeps only %d suppliers from %d!' % (
                    blockNum, succesData, self.eccmap.nodes()))
            if succesParity < self.eccmap.DataNeeded():
                dhnio.Dprint(4, 'backup.blockResult WARNING block %d-Parity keeps only %d suppliers from %d!' % (
                    blockNum, succesParity, self.eccmap.nodes()))
            dhnio.Dprint(6, 'backup.blockResult %d data[%d/%d] parity[%d/%d] %d more blocks' % (
                blockNum, succesData, self.eccmap.nodes(), succesParity, self.eccmap.nodes(), len(self.blocksResultDict)))


    def abort(self):
        dhnio.Dprint(4, 'backup.abort id='+str(self.backupID))
        self.ask4abort = True
        self.result = 'abort'


    def SetPacketResultCallback(self, cb):
        self.packetResultCallback = cb
        
    def GetStats(self):
        return self.sendingStats 
        





#!/usr/bin/python
#automat.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys

from twisted.internet import reactor
from twisted.internet.task import LoopingCall


_Counter = 0
_Index = {}
_Objects = {}
_StateChangedCallback = None


def get_new_index():
    global _Counter
    _Counter += 1
    return _Counter - 1


def create_index(name):
    global _Index
    id = name
    if _Index.has_key(id):
        i = 0
        while _Index.has_key(id+str(i)):
            i += 1
        id = name + str(i)
    _Index[id] = get_new_index()
    return id, _Index[id]


def set_object(index, obj):
    global _Objects
    _Objects[index] = obj
   
    
def clear_object(index):
    global _Objects
    if _Objects is None:
        return
    if _Objects.has_key(index):
        del _Objects[index]


def objects():
    global _Objects
    return _Objects


class Automat(object):
    timers = {}
    state = 'NOT_EXIST'
    def __init__(self, name, state, debug_level=18):
        self.id, self.index = create_index(name)
        self.name = name
        self.state = state
        self.debug_level = debug_level
        self._timers = {}
        self.init()
        self.startTimers()
        self.log(self.debug_level,  'new %s created with index %d' % (str(self), self.index))
        set_object(self.index, self)

    def __del__(self):
        global _Index
        global _StateChangedCallback
        id = self.id
        name = self.name
        debug_level = self.debug_level
        if _Index is None:
            self.log(debug_level, 'automat.__del__ Index is None')
            return
        index = _Index.get(id, None)
        if index is None:
            self.log(debug_level, 'automat.__del__ WARNING %s not found' % id)
            return
        del _Index[id]
        #clear_object(index)
        self.log(debug_level, '%s [%d] destroyed' % (id, index))
        if _StateChangedCallback is not None:
            _StateChangedCallback(index, id, name, '')

    def __repr__(self):
        return '%s[%s]' % (self.id, self.state)

    def init(self):
        pass

    def state_changed(self, oldstate, newstate):
        pass

    def A(self, event, arg):
        raise NotImplementedError

    def automat(self, event, arg=None):
        reactor.callLater(0, self.event, event, arg)

    def event(self, event, arg=None):
        global _StateChangedCallback
        self.log(self.debug_level * 4, '%s fired with event "%s"' % (self, event))# , sys.getrefcount(Automat)))
        old_state = self.state
        self.A(event, arg)
        new_state = self.state
        if old_state != new_state:
            self.stopTimers()
            self.state_changed(old_state, new_state)
            self.log(self.debug_level, '%s(%s): [%s]->[%s]' % (self.id, event, old_state, new_state))
            self.startTimers()
            if _StateChangedCallback is not None:
                _StateChangedCallback(self.index, self.id, self.name, new_state)

    def timer_event(self, name, interval):
        if self.timers.has_key(name) and self.state in self.timers[name][1]:
            self.automat(name)
        else:
            self.log(self.debug_level, '%s.timer_event ERROR timer %s not found in self.timers')

    def stopTimers(self):
        for name, timer in self._timers.items():
            if timer.running:
                timer.stop()
                self.log(self.debug_level * 4, '%s.stopTimers timer %s stopped' % (self, name))
        self._timers.clear()

    def startTimers(self):
        for name, (interval, states) in self.timers.items():
            if len(states) > 0 and self.state not in states:
                continue
            self._timers[name] = LoopingCall(self.timer_event, name, interval)
            self._timers[name].start(interval, False)
            self.log(self.debug_level * 4, '%s.startTimers timer %s started' % (self, name))

    def restartTimers(self):
        self.stopTimers()
        self.startTimers()

    def log(self, level, text):
        try:
            from dhnio import Dprint
            Dprint(level, text)
        except:
            try:
                from lib.dhnio import Dprint
                Dprint(level, text)
            except:
                pass

def SetStateChangedCallback(cb):
    global _StateChangedCallback
    _StateChangedCallback = cb








#!/usr/bin/env python
#central_connector.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in central_connector.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall


import lib.dhnio as dhnio
import lib.misc as misc
import lib.packetid as packetid
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control
from lib.automat import Automat


import lib.automats as automats
import p2p_connector
import central_service
import money

import identitypropagate


_CentralConnector = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _CentralConnector
    if _CentralConnector is None:
        _CentralConnector = CentralConnector('central_connector', 'AT_STARTUP', 4)
    if event is not None:
        _CentralConnector.automat(event, arg)
    return _CentralConnector

class CentralConnector(Automat):
    timers = {'timer-1hour':  (60*60, ['CONNECTED', 'DISCONNECTED', 'ONLY_ID']),
              'timer-30sec':  (30,    ['IDENTITY', 'REQUEST_SETTINGS', 'SETTINGS', 'SUPPLIERS',]),
              'timer-10sec':  (10,    ['IDENTITY'])}
    flagSettings = False

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('CENTRAL ' + newstate)
        p2p_connector.A('central_connector.state', newstate)

    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'propagate':
                self.flagSettings = False
                self.doInitCentralService()
                self.doSendIdentity()
                self.state = 'IDENTITY'
        #---IDENTITY---
        elif self.state == 'IDENTITY':
            if event == 'identity-ack' and not self.isSettingsExist():
                self.doSendRequestSettings()
                self.state = 'REQUEST_SETTINGS'
            elif event == 'identity-ack' and self.isSettingsExist() and not self.flagSettings:
                self.doSendSettings()
                self.state = 'SETTINGS'
            elif event == 'identity-ack' and self.isSettingsExist() and self.flagSettings and not self.isSuppliersNeeded():
                self.doSendBandWidthReport()
                self.state = 'CONNECTED'
            elif event == 'identity-ack' and self.isSettingsExist() and self.flagSettings and self.isSuppliersNeeded():
                self.doSendRequestSuppliers()
                self.state = 'SUPPLIERS'
            elif event == 'timer-10sec':
                self.doSendIdentity()
            elif event == 'timer-30sec':
                self.state = 'DISCONNECTED'
        #---SETTINGS---
        elif self.state == 'SETTINGS':
            if event == 'settings-ack':
                self.flagSettings = True
                self.state = 'SUPPLIERS'
            elif event == 'list-suppliers':
                self.flagSettings = True
                self.doSendBandWidthReport()
                self.state = 'CONNECTED'
            elif event == 'propagate':
                self.doSendIdentity()
                self.state = 'IDENTITY'
            elif event == 'timer-30sec':
                self.state = 'ONLY_ID'
            elif event == 'settings':
                self.doSendSettings()
        #---REQUEST_SETTINGS---
        elif self.state == 'REQUEST_SETTINGS':
            if event == 'request-settings-ack' and self.isSuppliersNeeded():
                self.flagSettings = True
                self.state = 'SUPPLIERS'
            elif event == 'request-settings-ack' and not self.isSuppliersNeeded():
                self.flagSettings = True
                self.doSendBandWidthReport()
                self.state = 'CONNECTED'
            elif event == 'propagate':
                self.doSendIdentity()
                self.state = 'IDENTITY'
            elif event == 'timer-30sec':
                self.state = 'ONLY_ID'
        #---SUPPLIERS---
        elif self.state == 'SUPPLIERS':
            if event == 'list-suppliers':
                self.doSendBandWidthReport()
                self.state = 'CONNECTED'
            elif event == 'propagate':
                self.doSendIdentity()
                self.state = 'IDENTITY'
            elif event == 'timer-30sec':
                self.state = 'ONLY_ID'
        #---ONLY_ID---
        elif self.state == 'ONLY_ID':
            if event in ['timer-1hour', 'propagate', 'settings', 'identity-ack', 'list-suppliers', 'list-customers']:
                self.doSendIdentity()
                self.state = 'IDENTITY'
        #---CONNECTED---
        elif self.state == 'CONNECTED':
            if event in ['propagate', 'timer-1hour']:
                self.doSendIdentity()
                self.state = 'IDENTITY'
            elif event == 'settings':
                self.doSendSettings()
                self.flagSettings = False
                self.state = 'SETTINGS'
        #---DISCONNECTED---
        elif self.state == 'DISCONNECTED':
            if ( event in ['timer-1hour', 'propagate', 'settings', 'identity-ack', 'list-suppliers', 'list-customers'] ) or ( event == 'p2p_connector.state' and arg is 'CONNECTED' ):
                self.flagSettings = False
                self.doSendIdentity()
                self.state = 'IDENTITY'

    def isSettingsExist(self):
        return settings.getCentralNumSuppliers() > 0

    def isSuppliersNeeded(self):
        return settings.getCentralNumSuppliers() <= 0 or \
               contacts.numSuppliers() != settings.getCentralNumSuppliers()

    def _saveRequestedSettings(self, newpacket):
        sd = dhnio._unpack_dict(newpacket.Payload)
        settings.uconfig().set('central-settings.needed-megabytes', sd.get('n', str(settings.DefaultNeededMb())+'Mb'))
        settings.uconfig().set('central-settings.shared-megabytes', sd.get('d', str(settings.DefaultDonatedMb())+'Mb'))
        settings.uconfig().set('central-settings.desired-suppliers', sd.get('s', '2'))
        settings.uconfig().set('emergency.emergency-email', sd.get('e1', ''))
        settings.uconfig().set('emergency.emergency-phone', sd.get('e2', ''))
        settings.uconfig().set('emergency.emergency-fax', sd.get('e3', ''))
        settings.uconfig().set('emergency.emergency-text', sd.get('e4', '').replace('<br>', '\n'))
        settings.uconfig().update()
        reactor.callLater(0, self.automat, 'request-settings-ack', newpacket)

    def doInitCentralService(self):
        central_service.init()

    def doSendIdentity(self):
        transport_control.RegisterInterest(
            lambda packet: self.automat('identity-ack', packet),
            settings.CentralID(),
            central_service.SendIdentity(True))

    def doSendSettings(self):
        packetID = packetid.UniqueID()
        transport_control.RegisterInterest(
            self.settingsAck,
            settings.CentralID(),
            packetID)
        central_service.SendSettings(True, packetID)

    def doSendRequestSettings(self):
        transport_control.RegisterInterest(
            lambda packet: self._saveRequestedSettings(packet),
            settings.CentralID(),
            central_service.SendRequestSettings(True))

    def doSendRequestSuppliers(self):
        return central_service.SendRequestSuppliers()

    def settingsAck(self, packet):
        try:
            status, last_receipt, = packet.Payload.split('\n', 2)
            last_receipt = int(last_receipt)
        except:
            status = 'error'
            last_receipt = -1
        dhnio.Dprint(4, 'central_connector.settingsAck [%s] last_receipt=%d' % (status, last_receipt))
        missing_receipts = money.SearchMissingReceipts(last_receipt)
        if len(missing_receipts) > 0:
            reactor.callLater(0, central_service.SendRequestReceipt, missing_receipts)
        self.automat('settings-ack', packet)

    def doSendBandWidthReport(self):
        central_service.LoopSendBandwidthReport()
        
        



#!/usr/bin/python
#raidread.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

#
# This is to read some possibly incomplete raid data and recover original data.
#
# Basic idea is to find a parity that is only missing one data, then fix that
#  data, and look at parities again to see if another is now missing only one.
#  This can be called an "iterative algorithm" or "belief propagation".
#  There are algorithms that can recover data when this can not, but
#    they are more complicated.
# Input is a file and a RAID spec.
# The spec says for each datablock which parity blocks XOR it.
# As we read in each datablock we XOR all the parity blocks for it.
# Probably best to read in all the datablocks at once, and do all
# the parities one time.  If we did one datablock after another
# the parity blocks would need to be active all the time.
#
# If we XOR integers, we have a byte order issue probably, though
# maybe not since they all stay the same order, whatever that is.
#
# Some machines are 64-bit.  Would be fun to make use of that if
# we have it.
#
# 2^1 => 3       ^ is XOR operator  - so maybe we can do it in Python
#
# In the production code, when there are multiple errors we really want
# to fix the one we can fix fastest first.  The only danger is that
# we get too many errors at the same time.  We reduce this danger if
# we fix the one we can fix fastest first.


import struct
import os
import sys


import lib.eccmap as eccmap
import lib.misc as misc
import lib.dhnio as dhnio
import lib.settings as settings
import lib.tmpfile as tmpfile

#------------------------------------------------------------------------------ 

def unpack_nbyte(number):
    result = long(0)
    for i in range(0,len(number)):
        result = 256*result + ord(number[i])
    return result

def RebuildOne(inlist, listlen, outfilename):
    RebuildOne_new(inlist, listlen, outfilename)

#rebuildOne_new, new2, and orig are just for debugging purposes
def RebuildOne_new(inlist, listlen, outfilename):
##    print "inlist=" + str(inlist)
##    print "inlist=" + str(listlen)
##    print "We can rebuild one " + outfilename + "\n"

    readsize = 1 # vary from 1 byte to 4 bytes

    raidfiles = range(listlen)   # just need a list of this size
    raidreads = range(listlen)
    for filenum in range(listlen):
        raidfiles[filenum] = open(inlist[filenum], "rb")
    rebuildfile = open(outfilename,"wb")

    loopcount = 0
    debugLoopcount = 2000
    debugI = 2000
    while 1:
#        if loopcount > debugLoopcount:
#            print "loopcount=" + str(loopcount)
        for k in range(listlen):
            raidreads[k] = raidfiles[k].read(2048)
#            if loopcount > debugLoopcount:
#                print "file" + str(i) + ", " + inlist[i] + " len=" + str(len(raidreads[i]))
        if not raidreads[0]:
            break
        i = 0
        while i < len(raidreads[0]):
            xor = 0
#            if loopcount > debugLoopcount and i > debugI:
#                print "i=" + str(i) + ", len=" + str(len(raidreads[0]))
#                print "listlen=" + str(listlen)
            for j in range(listlen):
#                if loopcount > debugLoopcount and i > debugI:
#                    print str(len(raidreads[j]))
                b1 = ord(raidreads[j][i])
                xor = xor ^ b1
#                if loopcount > debugLoopcount and i > debugI:
#                    print "j=" + str(j) + ", val=" + str(b1)
#            if loopcount > debugLoopcount and i>debugI:
#                print "loopcount=" + str(loopcount) + ", i=" + str(i) + ", len=" + str(len(raidreads[0]))

            rebuildfile.write(chr(xor))
            i += readsize
#        loopcount += 1

    for filenum in range(listlen):
        raidfiles[filenum].close()
        
    rebuildfile.close()


def RebuildOne_new2(inlist, listlen, outfilename):
    INTSIZE=settings.IntSize()
    fds = range(0,listlen)   # just need a list of this size
##    print "inlist=" + str(inlist)
##    print "inlist=" + str(listlen)
##    print "We can rebuild one " + outfilename + "\n"
    wholefile = dhnio.ReadBinaryFile(inlist[0])
    seglength = len(wholefile)   # just needed length of file
    for filenum in range(0, listlen):
        fds[filenum]=open(inlist[filenum],"r")
    fout = open(outfilename,"w")
    for i in range(0,seglength):
        xor = 0
        for j in range(0,listlen):
##            print "reading byte " + str(i) + " in file " + inlist[j]
            b1 = ord(fds[j].read(1))
##            print " val=" + str(b1)
            xor = xor ^ b1
        fout.write(chr(xor))
    for filenum in range(0, listlen):
        fds[filenum].close

# We XOR list of listlen input files and write result to a file named outfilename
def RebuildOne_orig(inlist, listlen, outfilename):
        INTSIZE=settings.IntSize()
        fds = range(0,listlen)   # just need a list of this size
##        print "We can rebuild one " + outfilename + "\n"
        wholefile = dhnio.ReadBinaryFile(inlist[0])
        seglength=len(wholefile)   # just needed length of file
        for filenum in range(0, listlen):
                fds[filenum]=open(inlist[filenum],"rb")
        fout=open(outfilename,"w")
        for i in range(0,seglength/INTSIZE):
                xor=0
                for j in range(0, listlen):
                        bstr1 = fds[j].read(INTSIZE)
                        #b1 = unpack_nbyte(bstr1)
                        b1, = struct.unpack(">l", bstr1)
                        xor = xor ^ b1
                outstr = struct.pack(">l", xor)
                fout.write(outstr)
        for filenum in range(0, listlen):
                fds[filenum].close


# If segment is good, there is a file for it, if not then no file exists.
# We only rebuild data segments.
# Could only make parity segments from existing data segments, so no help toward getting data.
# As long as we could rebuild one more data segment in a pass,
#  we do another pass to see if we are then able to do another.
#  When we can't do anything more we could how many good data segments there
#    are, and if we have all we win, if not we fail.

def raidread(OutputFileName, eccmapname, backupId, blockNumber, data_parity_dir=None):
    if data_parity_dir is None:
        # data_parity_dir = tmpfile.subdir('data-par')
        data_parity_dir = settings.getLocalBackupsDir()

    INTSIZE = settings.IntSize()
    myeccmap = eccmap.eccmap(eccmapname)
    GoodFiles = range(0, 200)

    MakingProgress = 1
    while MakingProgress == 1:
        MakingProgress = 0
        for PSegNum in range(myeccmap.paritysegments):
                PFileName = os.path.join(data_parity_dir, backupId + '-' + str(blockNumber) + '-' + str(PSegNum) + '-Parity')
                
                if os.path.exists(PFileName):
                        Map = myeccmap.ParityToData[PSegNum]
                        TotalDSegs = 0
                        GoodDSegs = 0
                        for DSegNum in Map:
                                TotalDSegs+=1
                                FileName = os.path.join(data_parity_dir, backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data')
                                if os.path.exists(FileName):
                                        GoodFiles[GoodDSegs]=FileName
                                        GoodDSegs+=1
                                else:
                                        BadName=FileName
                        if GoodDSegs == TotalDSegs - 1:
##                                print "We can rebuild one " + BadName + "\n"
                                MakingProgress=1
                                GoodFiles[GoodDSegs]=PFileName
                                GoodDSegs+=1
                                RebuildOne(GoodFiles,GoodDSegs,BadName)

    #  Count up the good segments and combine
    GoodDSegs = 0
    output = open(OutputFileName,"wb")
    for DSegNum in range(myeccmap.datasegments):
        FileName = os.path.join(data_parity_dir, backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data')
        if os.path.exists(FileName):
                GoodDSegs+=1
                moredata=open(FileName,"rb").read()
                output.write(moredata)
    output.close()
##    if(GoodDSegs == myeccmap.datasegments):
##        print "We got them all"
##    else:
##        print "We could not make all data segments"


def main():
    if (len(sys.argv) < 3):
        print "raidread needs an output filename and eccmap name"
        sys.exit(2)

    OutputFileName = sys.argv[1]
    eccmapname = sys.argv[2]

    print "raidread is starting with"
    print OutputFileName
    print eccmapname
    raidread(OutputFileName, eccmapname)


if __name__ == "__main__":
    main()





#!/usr/bin/python
#userconfig.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#
#to add a single item in user configuration:
#1. edit userconfig.py: UserConfig.default_xml_src
#   do not forget to add tag "label" and "info".
#2. edit settings.py
#   you can add access functions like getECC() or getCentralNumSuppliers()
#3. edit guisettings.py if you wish user to be able to edit this item:
#   add key pair to dictionary CSettings.items
#   you can define your own Widget (for example: XMLTreeEMailSettingsNode or XMLTreeQ2QSettingsNode)
#   if you are using ComboBox Widget (xmltreenodes.XMLTreeComboboxNode)
#   you need to add his definition after line comment "#define combo boxes"
#4. finally, you need to read this value using lib.settings and do something...


import os
import locale

from xml.dom import minidom
from xml.dom import Node
from xml.dom.minidom import getDOMImplementation

#-------------------------------------------------------------------------------

InfosDict = {
    'general':                      "General options.",
    'general-backups':              'How many backups of each directory to keep. ( 0 = unlimited )',
    'general-autorun':              "Starting the application during system startup.",
    'general-display-mode':         "Specifies how you want the window to display when you start the software.",
    'general-desktop-shortcut':     "Place shortcut on the Desktop?",
    'general-start-menu-shortcut':  "Add shortcut to the Start Menu?",
    'general-local-backups-enable': "Keep a copy of your backups on local HDD?",
    'updates':                      "Software updates options.",
    'updates-mode':                 "You can choose one of the install modes. Software must be restarted after installation of the new version.",
    'updates-shedule':              "You can setup updating schedule here.",
    'central-settings':             "Central Server Settings. Here you can manage your settings stored on the DataHaven.NET Central Server (c).",
    'desired-suppliers':            "Number of remote suppliers which keeps your backups.",
    'needed-megabytes':             "How many megabytes you need to store your files?",
    'shared-megabytes':             "How many megabytes you ready to donate to other users?",
    'folder-backups':               "Place for your local backups files.",
    'folder-restore':               'Location where your restored files should be placed.',
    'folder-customers':             'Place for donated space, other users will keep their files here.',
    'emergency':                    "We can contact you if your account balance is running low, if your backups are not working, or if your machine appears to not be working.",
    'emergency-first':              "What is your preferred method for us to contact you if there are problems?",
    'emergency-second':             "What is the second best method to contact you?",
    'emergency-email':              "What email address should we contact you at? Email contact is free.",
    'emergency-phone':              "If you would like to be contacted by phone, what number can we reach you at? ($1 per call for our time and costs)",
    'emergency-fax':                "If you would like to be contacted by fax, what number can we reach you at? ($0.50 per fax for our time and costs)",
    'emergency-text':               "Other method we should contact you by? Cost will be based on our costs.",
    'network':                      'Network settings.',
    'network-send-limit':           'The value in kilobytes per second to decrease network load. At the moment the maximum sending speed that DataHaven.NET can support is about half megabyte per second. 0 - no limit.',
    'network-receive-limit':        'Limit incoming traffic with a value in kilobytes per second. 0 - no limit.',
    'transport':                    'You can use different protocols to transfer packets, called "transports". Here you can customize your transport settings.',
    'transport-tcp':                "transport_tcp uses the standard TCP protocol to transfer packets.",
    'transport-tcp-port':           "Enter the TCP port number for the transport_tcp, it will be used to connect with your machine.",
    'transport-tcp-enable':         "transport_tcp uses the standard TCP protocol to transfer packets.<br>Do you want to use transport_tcp?",
    'transport-udp':                'transport_udp provides NAT traversing functionality',
    'transport-udp-enable':         'transport_udp provides NAT traversing functionality,<br>Do you want to use transport_udp?',
    'transport-udp-port':           'Enter the UDP port number for the transport_tcp, it will be used to connect with your machine. Enter 0 and your OS will choose the port automatically.',
    'transport-ssh':                "transport_ssh uses the technique of public and private key for the protection of transmitted packets.",
    'transport-ssh-port':           "Enter the port number for the transport_ssh, which will be used to connect with you.",
    'transport-ssh-enable':         "Enable transport_ssh?",
    'transport-cspace':             'CSpace provides a platform for secure, decentralized, user-to-user communication over the internet.<br>Go to <a href="http://cspace.in" target=_blank>http://cspace.in</a> for more details.',
    'transport-cspace-enable':      'CSpace provides a platform for secure, decentralized, user-to-user communication over the internet.<br>Go to <a href="http://cspace.in" target=_blank>http://cspace.in</a> for more details.<br>Do you want to use transport_cspace?',
    'transport-cspace-key-id':      'CSpace provides a platform for secure, decentralized, user-to-user communication over the internet.<br>Go to <a href="http://cspace.in" target=_blank>http://cspace.in</a> for more details.<br>Enter your CSpace user id here.',
    'upnp-enabled':                 'Do you want to use UPnP to configure port forwarding?',
    'gui':                          "Here You can setup user interfce options.",
    'gui-font-size':                'Specify the font size. Possible values: 6 - 56',
    'debug-level':                  "Higher values will produce more log messages.",
    'stream-enable':                "Go to http://127.0.0.1:[logs port number] to browse the program log.",
    'memdebug-enable':              'Go to http://127.0.0.1:[memdebug port number] to see memory usage.',
    'memprofile-enable':            'Use guppy to profile memory usage.',
    }

LabelsDict = {
    'general':                              'general',
    'general-backups':                      'backup copies',
    'general-local-backups-enable':         'local backups',
    'general-wait-suppliers-enable':        'wait suppliers',
    'general-autorun':                      'autorun',
    'general-display-mode':                 'display mode',
    'general-desktop-shortcut':             'desktop shortcut',
    'general-start-menu-shortcut':          'start menu shortcut',
    'updates':                              'updates',
    'updates-mode':                         'mode',
    'updates-shedule':                      'schedule',
    'central-settings':                     'central server',
    'desired-suppliers':                    'number of suppliers',
    'needed-megabytes':                     'needed space',
    'shared-megabytes':                     'donated space',
    'folder':                               'folders',
    'folder-customers':                     'donated space',
    'folder-backups':                       'local backups',
    'folder-restore':                       'restored files',
    'folder-messages':                      'messages',
    'folder-receipts':                      'receipts',
    'folder-temp':                          'temporary files',
    'emergency':                            'emergency',
    'emergency-first':                      'primary',
    'emergency-second':                     'secondary',
    'emergency-email':                      'email',
    'emergency-phone':                      'phone',
    'emergency-fax':                        'fax',
    'emergency-text':                       'other',
    'network':                              'network',
    'network-send-limit':                   'outgoing bandwidth limit',
    'network-receive-limit':                'incoming bandwidth limit',
    'transport':                            'transports',
    'transport-tcp':                        'transport_tcp',
    'transport-tcp-port':                   'TCP port',
    'transport-tcp-enable':                 'transport_tcp enable',
    'transport-udp':                        'transport_udp',
    'transport-udp-port':                   'UDP port',
    'transport-udp-enable':                 'transport_udp enable',
    'transport-ssh':                        'ssh',
    'transport-ssh-port':                   'SSH port',
    'transport-ssh-enable':                 'transport_ssh enable',
    'transport-q2q':                        'q2q',
    'transport-q2q-host':                   'q2q server address',
    'transport-q2q-username':               'q2q username',
    'transport-q2q-password':               'q2q password',
    'transport-q2q-enable':                 'q2q enabled',
    'transport-email':                      'email',
    'transport-email-address':              'address',
    'transport-email-pop-host':             'pop server hostname',
    'transport-email-pop-port':             'pop server port',
    'transport-email-pop-username':         'pop username',
    'transport-email-pop-password':         'pop password',
    'transport-email-pop-ssl':              'pop security',
    'transport-email-smtp-host':            'smtp server hostname',
    'transport-email-smtp-port':            'smtp server port',
    'transport-email-smtp-username':        'smtp username',
    'transport-email-smtp-password':        'smtp password',
    'transport-email-smtp-need-login':      'smtp authorization',
    'transport-email-smtp-ssl':             'smtp security',
    'transport-email-enable':               'enabled',
    'transport-http':                       'http',
    'transport-http-server-port':           'server port',
    'transport-http-ping-timeout':          'ping timeout',
    'transport-http-enable':                'enabled',
    'transport-http-server-enable':         'server enabled',
    'transport-skype':                      'skype',
    'transport-skype-enable':               'enabled',
    'transport-cspace':                     'transport_cspace',
    'transport-cspace-enable':              'transport_cspace enable',
    'transport-cspace-key-id':              'CSpace user id',
    'gui':                                  'user interface',
    'gui-font-size':                        'font size',
    'logs':                                 'logs',
    'debug-level':                          'debug level',
    'stream-enable':                        'enable logs',
    'stream-port':                          'logs port number',
    'memdebug-enable':                      'enable memdebug',
    'memdebug-port':                        'memdebug port number',
    'memprofile-enable':                    'enable memory profiler',
    'traffic-enable':                       'enable packets traffic',
    'traffic-port':                         'traffic port number',
    'other':                                'other',
    'upnp-enabled':                         'UPnP enable',
    'upnp-at-startup':                      'check UPnP at startup',
    }

#------------------------------------------------------------------------------ 

class UserConfig:
    default_xml_src = ur"""<settings>
 <general>
  <general-backups>
   2
  </general-backups>
  <general-local-backups-enable>
   True
  </general-local-backups-enable>
  <general-wait-suppliers-enable>
   True
  </general-wait-suppliers-enable>
  <general-autorun>
   True
  </general-autorun>
  <general-display-mode>
   iconify window
  </general-display-mode>
  <general-desktop-shortcut>
   True
  </general-desktop-shortcut>
  <general-start-menu-shortcut>
   True
  </general-start-menu-shortcut>
 </general>
 <updates>
  <updates-mode>
   install automatically
  </updates-mode>
  <updates-shedule>
1
12:00:00
6

  </updates-shedule>
 </updates>
 <central-settings>
  <desired-suppliers>
   4
  </desired-suppliers>
  <needed-megabytes>
   4GB
  </needed-megabytes>
  <shared-megabytes>
   8GB
  </shared-megabytes>
 </central-settings>
 <folder>
  <folder-customers>

  </folder-customers>
  <folder-backups>

  </folder-backups>
  <folder-restore>

  </folder-restore>
  <folder-messages>

  </folder-messages>
  <folder-receipts>

  </folder-receipts>
  <folder-temp>

  </folder-temp>
 </folder>
 <emergency>
  <emergency-first>
   email
  </emergency-first>
  <emergency-second>
   phone
  </emergency-second>
  <emergency-email>

  </emergency-email>
  <emergency-phone>

  </emergency-phone>
  <emergency-fax>

  </emergency-fax>
  <emergency-text>

  </emergency-text>
 </emergency>
 <network>
  <network-proxy>
   <network-proxy-enable>
    False
   </network-proxy-enable>
   <network-proxy-host>

   </network-proxy-host>
   <network-proxy-port>

   </network-proxy-port>
   <network-proxy-username>

   </network-proxy-username>
   <network-proxy-password>

   </network-proxy-password>
   <network-proxy-ssl>
    False
   </network-proxy-ssl>
  </network-proxy>
  <network-send-limit>
   0
  </network-send-limit>
  <network-receive-limit>
   0
  </network-receive-limit>
 </network>
 <transport>
  <transport-tcp>
   <transport-tcp-port>
    7771
   </transport-tcp-port>
   <transport-tcp-enable>
    True
   </transport-tcp-enable>
  </transport-tcp>
  <transport-udp>
   <transport-udp-port>
    0
   </transport-udp-port>
   <transport-udp-enable>
    True
   </transport-udp-enable>
  </transport-udp>
  <transport-ssh>
   <transport-ssh-port>
    5022
   </transport-ssh-port>
   <transport-ssh-enable>
    False
   </transport-ssh-enable>
  </transport-ssh>
  <transport-q2q>
   <transport-q2q-host>

   </transport-q2q-host>
   <transport-q2q-username>

   </transport-q2q-username>
   <transport-q2q-password>

   </transport-q2q-password>
   <transport-q2q-enable>
    True
   </transport-q2q-enable>
  </transport-q2q>
  <transport-email>
   <transport-email-address>

   </transport-email-address>
   <transport-email-pop-host>

   </transport-email-pop-host>
   <transport-email-pop-port>

   </transport-email-pop-port>
   <transport-email-pop-username>

   </transport-email-pop-username>
   <transport-email-pop-password>

   </transport-email-pop-password>
   <transport-email-pop-ssl>
    False
   </transport-email-pop-ssl>
   <transport-email-smtp-host>

   </transport-email-smtp-host>
   <transport-email-smtp-port>

   </transport-email-smtp-port>
   <transport-email-smtp-username>

   </transport-email-smtp-username>
   <transport-email-smtp-password>

   </transport-email-smtp-password>
   <transport-email-smtp-need-login>
    False
   </transport-email-smtp-need-login>
   <transport-email-smtp-ssl>
    False
   </transport-email-smtp-ssl>
   <transport-email-enable>
    False
   </transport-email-enable>
  </transport-email>
  <transport-http>
   <transport-http-server-port>
    9786
   </transport-http-server-port>
   <transport-http-ping-timeout>
    5
   </transport-http-ping-timeout>
   <transport-http-enable>
    True
   </transport-http-enable>
   <transport-http-server-enable>
    True
   </transport-http-server-enable>
  </transport-http>
  <transport-skype>
   <transport-skype-enable>
    False
   </transport-skype-enable>
  </transport-skype>
  <transport-cspace>
   <transport-cspace-enable>
    True
   </transport-cspace-enable>
   <transport-cspace-key-id>

   </transport-cspace-key-id>
  </transport-cspace>
 </transport>
 <gui>
  <gui-font-size>
   12
  </gui-font-size>
 </gui>
 <logs>
  <debug-level>
   4
  </debug-level>
  <stream-enable>
   False
  </stream-enable>
  <stream-port>
   9999
  </stream-port>
  <traffic-enable>
   False
  </traffic-enable>
  <traffic-port>
   9997
  </traffic-port>
  <memdebug-enable>
   False
  </memdebug-enable>
  <memdebug-port>
   9996
  </memdebug-port>
  <memprofile-enable>
   False
  </memprofile-enable>
 </logs>
 <other>
  <upnp-enabled>
   True
  </upnp-enabled>
  <upnp-at-startup>
   False
  </upnp-at-startup>
 </other>
</settings>"""

#  <memdebug-enable label="memdebug enable">
#   False
#  </memdebug-enable>
#  <memdebug-port label="memdebug port number">
#   9995
#  </memdebug-port>

    xmlsrc = ''
    data = {}
    labels = {}
    infos = {}
    default_data = {}
    default_order = []

    def __init__(self, filename):
        self.filename = filename
        if os.path.isfile(self.filename):
            self._read()
        else:
            self._create()

        doc1 = self._parse(self.default_xml_src)
        self._load(
            self.default_data,
            doc1.documentElement,
            order=self.default_order, )

        doc2 = self._parse(self.xmlsrc)
        self._load(
            self.data,
            doc2.documentElement)

        self._validate(True)


    def _parse(self, src):
        try:
            s = src.encode('utf-8')
            return minidom.parseString(s)
        except:
            return minidom.parseString(self.default_xml_src.encode('utf-8'))

    def _read(self):
        fin = open(self.filename, 'r')
        src = fin.read()
        fin.close()
##        print '_read before', type(src)
        self.xmlsrc = src.decode(locale.getpreferredencoding())
##        print '_read after', type(self.xmlsrc)

    def _write(self):
##        print '_write before', type(self.xmlsrc)
        src = self.xmlsrc.encode(locale.getpreferredencoding())
##        print '_write after', type(src)
        try:
            fout = open(self.filename, 'w')
            fout.write(src)
            fout.flush()
            os.fsync(fout.fileno())
            fout.close()
        except:
            pass

    def _create(self):
        self.xmlsrc = self.default_xml_src
        self._write()

    # check existing user-config and our template, add nodes if they are missing
    def _validate(self, remove=False):
        changed = False
        for key in self.default_data.keys():
            if not self.data.has_key(key):
                self.data[key] = self.default_data[key]
                changed = True
        if remove:
            for key in self.data.keys():
                if not self.default_data.has_key(key):
                    del self.data[key]
                    changed = True
        if changed:
            self.xmlsrc = self._make_xml()[0]
            self._write()

    def _load(self, data, node, path='', order=None):
        d = get_text(node)
        if path != '':
            data[path] = d
        if order is not None:
            order.append(path)

        if not self.labels.has_key(path):
            l = get_label(node)
            if l is not None:
                self.labels[path] = l
            else:
                self.labels[path] = node.tagName
#        else:
#            set_label(node, self.labels[path])

        if not self.infos.has_key(path):
            i = get_info(node)
            if i is not None:
                self.infos[path] = i
        # else:
            # set_info(node, self.infos[path])

        for subnode in node.childNodes:
            if subnode.nodeType == Node.ELEMENT_NODE:
                name = str(subnode.tagName)
                if path != '':
                    name = path+'.'+name
                self._load(data, subnode, name, order)

    def _from_data(self, parent, doc):
        for path in self.default_order:
            if path.strip() == '':
                continue
            value = self.data.get(path, '')
            # label = self.labels.get(path, '')
            # info = self.infos.get(path, '')
            leafs = path.split('.')
            prevleaf = parent
            leafnode = None
            for leaf in leafs:
                leafnode = get_child(prevleaf, leaf)
                if leafnode is None:
                    leafnode = doc.createElement(leaf)
                    prevleaf.appendChild(leafnode)
                prevleaf = leafnode
            # set_label(leafnode, label)
            # set_info(leafnode, info)
            set_text(leafnode, value)

    def _make_xml(self):
        impl = getDOMImplementation()
        doc = impl.createDocument(None, 'settings', None)
        rootnode = doc.documentElement
        self._from_data(rootnode, doc)
        xmlsrc = doc.toprettyxml("  ","\n")
        return xmlsrc, rootnode

    def update(self, node=None):
        if node is None:
            self.xmlsrc = self.Serialize()
        else:
            self.UnserializeObject(node)
        self._write()

    def Serialize(self):
        doc1 = self._parse(self.xmlsrc)
        self.default_order = []
        self._load(
            self.default_data,
            doc1.documentElement,
            order=self.default_order, )
        return self._make_xml()[0]

    def SerializeObject(self):
        doc1 = self._parse(self.xmlsrc)
        self.default_order = []
        self._load(
            self.default_data,
            doc1.documentElement,
            order=self.default_order, )
        return self._make_xml()[1]

    def Unserialize(self, src):
        doc = self._parse(src)
        node = doc.documentElement
        self.data.clear()
        self._load(self.data, node)
        self.xmlsrc = doc.toprettyxml("  ", "\n") # doubles if put spaces here

    def UnserializeObject(self, xml_object):
        self.data.clear()
        self._load(self.data, xml_object)
        self.xmlsrc = xml_object.toprettyxml("  ", "\n") # doubles if put spaces here

    # use this when user needs to "reset to factory defaults"
    def reset(self):
        self.xmlsrc = self.default_xml_src

    def get(self, key, request=None):
        if not request:
            return self.data.get(key, None)
        elif request=='all':
            return (self.data.get(key, None),
                    self.labels.get(key, None),
                    self.infos.get(key, None),
                    self.default_data.get(key, None),)
        elif request=='data':
            return self.data.get(key, None)
        elif request=='label':
            return self.labels.get(key, None)
        elif request=='info':
            return self.infos.get(key, None)
        elif request=='default':
            return self.default_data.get(key, None)
        return self.data.has_key(key)

    def has(self, key):
        return self.data.has_key(key)

    def set(self, key, value, request=None):
        if request=='data':
            self.data[key] = value
        elif request=='label':
            self.labels[key] = value
        elif request=='info':
            self.infos[key] = value
        else:
            self.data[key] = value

    def get_childs(self, key, request=None):
        d = {}
        if request=='data':
            for k,v in self.data.items():
                if k.startswith(str(key)+'.'): d[k] = v
        elif request=='label':
            for k,v in self.labels.items():
                if k.startswith(str(key)+'.'): d[k] = v
        elif request=='info':
            for k,v in self.labels.items():
                if k.startswith(str(key)+'.'): d[k] = v
        elif request=='default':
            for k,v in self.default_data.items():
                if k.startswith(str(key)+'.'): d[k] = v
        else:
            for k,v in self.data.items():
                if k.startswith(str(key)+'.'): d[k] = v
        return d

    def set_childs(self, key, childs_dict):
        if not self.data.has_key(key):
            return
        for k,v in childs_dict.items():
            _key = k
            if not k.startswith(key+'.'):
                _key = key+'.'+k
            self.data[_key] = v

    def make_child_name(self, key, namebase):
        i = 0
        while True:
            name = unicode(key+'.'+namebase+str(i))
            if not self.data.has_key(name):
                return name
            i += 1

    def print_all(self):
        for path in self.default_order:
            if path.strip() == '':
                continue
            value = self.data.get(path, '')
            label = self.labels.get(path, '')
            print path.ljust(40),
            print label.ljust(20),
            print value

    def print_all_html(self):
        src = ''
        for path in self.default_order:
            if path.strip() == '':
                continue
            value = self.data.get(path, '')
            label = self.labels.get(path, '')
            info = self.infos.get(path, '')
            src += '<li><p><b>%s</b><br>\n' % label
            src += '&nbsp;' * 4 + '\n'
            src += info + '\n'
            src += '</p></li>\n'''
        return src

     

#-------------------------------------------------------------------------------

def get_child(father,childname):
    for son in father.childNodes:
        if son.nodeName == childname:
            return son
    return None

def get_text(xmlnode):
    rc = u''
    for node in xmlnode.childNodes:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data.strip()
    return rc.encode(locale.getpreferredencoding())

def set_text(xmlnode, txt):
    try:
        text = txt.decode(locale.getpreferredencoding())
    except:
        text = ''
    for node in xmlnode.childNodes:
        if node.nodeType == node.TEXT_NODE:
            node.data = u''
    j = 0
    while j < len(xmlnode.childNodes):
        node = xmlnode.childNodes[j]
        if node.nodeType == node.TEXT_NODE:
            node.data = text
            return
        j += 1
    node = xmlnode.ownerDocument.createTextNode(text)#.decode('latin_1'))
    xmlnode.appendChild(node)

#def get_label(xmlnode):
#    label = xmlnode.attributes.get('label', None)
#    if label is not None:
#        return label.value.encode(locale.getpreferredencoding())
#    return None

def get_label(xmlnode):
    global LabelsDict
    return LabelsDict.get(xmlnode.tagName, xmlnode.tagName)

#def set_label(xmlnode, label):
#    if label != '':
#        xmlnode.setAttribute('label', label.decode(locale.getpreferredencoding()))

#def get_info(xmlnode):
#    label = xmlnode.attributes.get('info', None)
#    if label is not None:
#        v = label.value
#        v = v.replace('\\n ', '\n')
#        v = v.replace('\\n', '\n')
#        return v.encode(locale.getpreferredencoding())
#    return None

#def set_info(xmlnode, info):
#    if info != '':
#        xmlnode.setAttribute('info', info.decode(locale.getpreferredencoding()))

def get_info(xmlnode):
    global InfosDict
    return InfosDict.get(xmlnode.tagName, '')

#-------------------------------------------------------------------------------

def main():
    import settings
    uc = UserConfig(settings.UserConfigFilename())
    uc.update()
    # import pprint
    # pprint.pprint(uc.data)
    print uc.print_all_html()

##    for path in uc.default_order:
##        if path.strip() == '':
##            continue
##        leafs = path.split('.')
##        print path, len(leafs)


if __name__ == "__main__":
    main()






#!/usr/bin/python
#events.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time


_OutputFunc = None


def init(output_func):
    global _OutputFunc
    _OutputFunc = output_func


def call(typ, module, message, text=''):
    global _OutputFunc
    if _OutputFunc is None:
        return
    _OutputFunc('event %s (((%s))) [[[%s]]] %s' % (typ, module, message, text))

def info(module, message, text=''):
    call('info', module, message, text)
    

def notify(module, message, text=''):
    call('notify', module, message, text)


def warning(module, message, text=''):
    call('warning', module, message, text)

    
   


#!/usr/bin/env python
#transport_ssh_client.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import stat

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in transport_ssh_client.py')

#from twisted.conch.client import agent
#from twisted.conch.client import connect
#from twisted.conch.client import default
#from twisted.conch.client import options
from twisted.conch.ssh import channel
#from twisted.conch.ssh import filetransfer
from twisted.conch.ssh import keys
from twisted.conch.ssh import userauth
from twisted.conch.ssh.connection import SSHConnection
from twisted.conch.ssh import common
#from twisted.conch.ssh import userauth
from twisted.internet import defer
from twisted.internet import protocol
from twisted.internet.interfaces import IConsumer
#from twisted.python.filepath import FilePath
#from twisted.python import log
#from twisted.protocols import basic

from zope.interface import implements


#import misc
import dhnio
import transport_control


##import datahaven.lib.identity as identity
##import datahaven.lib.settings as settings
##log.startLogging(sys.stderr)

def fail(err):
    dhnio.Dprint(1, 'transport_ssh.fail NETERROR ' + str(err.getErrorMessage()))
    raise err

class MySSHConnection(SSHConnection):
    def __init__(self, filename, stop_defer):
        SSHConnection.__init__(self)
        self.filename = filename
        self.stop_defer = stop_defer

    def serviceStarted(self):
        dhnio.Dprint(14,'transport_ssh.MySSHConnection.serviceStarted')
        self.openChannel(MySSHSession(self.filename, self.stop_defer))

class MyFileTransferClient(protocol.Protocol):
    BUFFER_SIZE = 16384

    def __init__(self, filename, stop_defer):
        self.filename = filename
        self.stop_defer = stop_defer
        self.buf = ''
        self.total_size = 0
        self.total_sent = 0

    def connectionMade(self):
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.connectionMade')
        self.buf = ''
        self.total_size = 0
        self.total_sent = 0
        try:
            self.total_size = os.stat(self.filename)[stat.ST_SIZE]
            self.fin = open(self.filename, 'rb')
            dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.connectionMade file %s have %s bytes' % (self.filename, str(self.total_size)))
        except:
            dhnio.Dprint(1, 'transport_ssh.MyFileTransferClient.connectionMade ERROR opening file: ' + str(self.filename))
            dhnio.DprintException()
            self.transport.loseConnection()
            return
        self.sendNextPart()

    def dataReceived(self, data):
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.dataReceived: '+str(data))
        try:
            amount = int(data)
        except:
            dhnio.Dprint(1, 'transport_ssh.MyFileTransferClient.dataReceived NETERROR bad response from the ssh server')
##            transport_control.connectionStatusCallback('ssh', self.transport.getPeer(), None, 'bad response from the ssh server')
            transport_control.sendStatusReport(self.transport.getPeer(),
                self.filename,
                'failed', 'ssh',
                None, 'bad response from the ssh server')
            self.breakTransfer()
            self.connectionLost(0)
            return

        if amount < self.total_size:
            self.sendNextPart()
        else:
            self.finishTransfer()
            self.connectionLost(1)

    def sendNextPart(self):
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.sendNextPart')
        self.buf = ''
        try:
            self.buf = self.fin.read(self.BUFFER_SIZE)
            self.fin.flush()
        except:
            dhnio.Dprint(1, 'transport_ssh.MyFileTransferClient ERROR reading buffer data from file')
            self.breakTransfer()
            self.connectionLost(0)
            return

##        if len(self.buf) == 0:
##            self.finishTransfer()
##            self.connectionLost(1)
##            return
##
        try:
            self.transport.write(self.buf)
        except:
            dhnio.Dprint(1, 'transport_ssh.MyFileTransferClient.sendNextPart NETERROR writing data to the transport')
##            transport_control.connectionStatusCallback('ssh', self.transport.getPeer(), None, 'error writing data to the transport')
            transport_control.sendStatusReport(self.transport.getPeer(),
                self.filename,
                'failed', 'ssh', None,
                'error writing data to the transport' )
            self.breakTransfer()
            self.connectionLost(0)
            return
        self.total_sent += len(self.buf)
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.sendNextPart total %s bytes sent' % str(self.total_sent))

    def finishTransfer(self):
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.finishTransfer')
        try:
            self.fin.close()
        except:
            pass

    def breakTransfer(self):
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.breakTransfer')
        try:
            self.fin.close()
        except:
            pass

    def connectionLost(self, reason):
        dhnio.Dprint(14, 'transport_ssh.MyFileTransferClient.connectionLost')
        try:
            self.fin.close()
        except:
            pass
        self.transport.loseConnection()
        self.stop_defer.callback(1)

class MySSHSession(channel.SSHChannel):
    implements(IConsumer)

    name = 'session'

    def __init__(self, filename, stop_defer):
        channel.SSHChannel.__init__(self)
        self.filename = filename
        self.stop_defer = stop_defer

    def registerProducer(self, producer, streaming):
        self.producer = producer
        if not streaming:
            self.producer.resumeProducing()

    def channelOpen(self, foo):
        d = self.conn.sendRequest(self, 'subsystem',
            common.NS(self.conn.options['subsystem']),
                wantReply=1)
        d.addCallback(self._cbSubsystem)
        d.addErrback(fail)

    def _cbSubsystem(self, result):
        self.client = MyFileTransferClient(self.filename, self.stop_defer)
        self.client.makeConnection(self)
        self.dataReceived = self.client.dataReceived

class MyUserAuth(userauth.SSHUserAuthClient):
    def __init__(self, user, instance, public_key_, private_key_):
        userauth.SSHUserAuthClient.__init__(self,user,instance)
        self.public_key = keys.Key.fromString( public_key_ )
        self.private_key = keys.Key.fromString( private_key_ )

    def getPublicKey(self):
        dhnio.Dprint(14,'transport_ssh.MyUserAuth.getPublicKey')
        return self.public_key.blob()

    def getPrivateKey(self):
        dhnio.Dprint(14,'transport_ssh.MyUserAuth.getPrivateKey')
        return defer.succeed(self.private_key.keyObject)

def my_verifyHostKey(transport, host, pubKey, fingerprint):
    return defer.succeed(1)

##class MySSHClientFactory(SSHClientFactory):
##    def __init__(self, d, options, verifyHostKey, userAuthObject):
##        SSHClientFactory.__init__(self,d,options,verifyHostKey,userAuthObject)





#!/usr/bin/python
#diskusage.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import time
import glob

import dhnio
import settings
import diskspace


if dhnio.Windows():
    import string
    import win32api
    import win32file

def GetWinDriveSpace(drive): # return free space in bytes, total space in bytes
    sectorsPerCluster, bytesPerSector, numFreeClusters, totalNumClusters = win32file.GetDiskFreeSpace(drive + ":\\")
    sectorsPerCluster = long(sectorsPerCluster)
    bytesPerSector = long(bytesPerSector)
    numFreeClusters = long(numFreeClusters)
    totalNumClusters = long(totalNumClusters)

    #print "Drive:     ", drive + ":\\"
    #print "FreeSpace: ", (numFreeClusters * sectorsPerCluster * bytesPerSector) / (1024 * 1024), "MB"
    #print "TotalSpace:", (totalNumClusters * sectorsPerCluster * bytesPerSector) / (1024 * 1024), "MB"
    #print "UsedSpace: ", ((totalNumClusters - numFreeClusters ) * sectorsPerCluster * bytesPerSector) / (1024 * 1024), "MB"
    return (numFreeClusters * sectorsPerCluster * bytesPerSector), (totalNumClusters * sectorsPerCluster * bytesPerSector)

def GetLinuxDriveSpace(path): # return free space in bytes, total space in bytes
    s = os.statvfs(path)
    return s.f_bsize * s.f_bavail, s.f_bsize*(s.f_blocks-s.f_bavail)

def GetDriveSpace(path):
    if dhnio.Windows():
        drive = os.path.abspath(path)[0]
        if os.path.isdir(drive+':'):
            # the drive the data directory is on, ie C
            driveFreeSpace, driveTotalSpace = GetWinDriveSpace(drive)
            return driveFreeSpace, driveTotalSpace
        else:
            return None, None
    else:
        # on linux the mount points can make a directory be off a different disk than root
        driveFreeSpace, driveTotalSpace = GetLinuxDriveSpace(path)
        return driveFreeSpace, driveTotalSpace

def SumFileSizes(fileList):
    fileSizeTotal = 0
    for filename in fileList:
        try:
            fileSizeTotal += os.path.getsize(filename)
        except:
            pass
    return fileSizeTotal

def GetOurTempFileSizeTotal(tempDirectory):
    ourFileMasks = ['*-Data', '*-Parity', '*dhn*', '*.controloutbox', 'newblock-*', '*.backup']
    ourFileSizes = 0
    for mask in ourFileMasks:
        ourFileSizes += SumFileSizes(glob.glob(os.path.join(tempDirectory, mask)))
    return ourFileSizes

def OkToShareSpace(desiredSharedSpaceMB):
    # make sure a user really has the space they claim they want to share
    dataDir = settings.getCustomersFilesDir()
    dataDriveFreeSpace, dataDriveTotalSpace = GetDriveSpace(dataDir)
    currentlySharedSpace = GetDirectorySize(dataDir)
    if (currentlySharedSpace + dataDriveFreeSpace/(1024*1024)) < desiredSharedSpaceMB:
        return False
    else:
        return True
    
def GetDirectorySize(directoryPath):
    return dhnio.getDirectorySize(directoryPath)/(1024*1024)

#------------------------------------------------------------------------------ 

def main():
    dataDir = settings.getCustomersFilesDir()
    tempDir = settings.TempDir()
    dataDriveFreeSpace = 0
    dataDriveTotalSpace = 0
    tempDriveFreeSpace = 0
    tempDriveTotalSpace = 0

    dataDriveFreeSpace, dataDriveTotalSpace = GetDriveSpace(dataDir)
    tempDriveFreeSpace, tempDriveTotalSpace = GetDriveSpace(tempDir)

    print "data dir =", dataDir
    print "tep dir =", tempDir
    print "data dir: " + str(dataDriveFreeSpace/(1024*1024)) +"MB free/" + str(dataDriveTotalSpace/(1024*1024)) +"MB total"
    print "temp dir: " + str(tempDriveFreeSpace/(1024*1024)) +"MB free/" + str(tempDriveTotalSpace/(1024*1024)) +"MB total"

    print time.time()
    print "our temp files: " + str(GetOurTempFileSizeTotal(tempDir)/(1024*1024)) + "MB"
    ourFileMasks = ['*-Data', '*-Parity', '*dhn*', '*.controloutbox', 'newblock-*', '*.backup']
    for mask in ourFileMasks:
        print time.time()
        print mask + "=" + str(SumFileSizes(glob.glob(os.path.join(tempDir, mask))))

    print time.time()

    GetDirectorySize(dataDir)

    ds = diskspace.DiskSpace()
    print ds.getValueBest(dataDriveFreeSpace)

    print "at OkToShareSpace ..."
    print "ok to share 100MB - should be true"
    print OkToShareSpace(100)
    print "ok to share 12345678MB - should be false"
    print OkToShareSpace(12345678)


if __name__ == '__main__':
    main()